%Load the optimal network for Sum Kp - S001 nowcasting,
%and store it as ASCII files that can be read by saaps.ann.WeightsCreator.
%Validation set gives:
% T1 = -7 days
% S1 = 3 hidden neurons

%Peter Wintoft 01-03-29.
%Update 01-09-05: anom002 -> anom001

%Load networks.
load /space/peter/AnomPred/S001/ffkp4

%Select optimal nowcasting model.
j=find([model.t2]==10);
m=model(j(1));
for i=2:length(j)
  if model(j(i)).b.rmse<m.b.rmse
    m=model(j(i));
  end
end
disp('-------------')
disp('Optimal network:')
disp(['Validation rmse = ' num2str(m.b.rmse)])
disp(['T1 = ' num2str(m.t1)])
disp(['T2 = ' num2str(m.t2)])
disp(['S1 = ' num2str(m.s1)])

%Get first layer weights and biases.
w1=m.net.iw{1};
b1=m.net.b{1};
[r1,c1]=size(w1);
disp('-------------')
disp('Layer 1')
disp([num2str(r1) ' neurons'])
disp([num2str(c1) ' inputs to each neuron'])
disp([m.net.layers{1}.transferfcn ' transfer function'])

%Get second layer weights and biases.
w2=m.net.lw{2,1};
b2=m.net.b{2};
[r2,c2]=size(w2);
disp('-------------')
disp('Layer 2')
disp([num2str(r2) ' neurons'])
disp([num2str(c2) ' inputs to each neuron'])
disp([m.net.layers{2}.transferfcn ' transfer function'])

%Store layer one.
fid=fopen('anom001l1.txt','w');
%Number of neurons and number of inputs.
fprintf(fid,'%d %d\n',r1,c1);
%Cycle over neurons.
for n=1:r1
  %Store weights for neuron n.
  fprintf(fid,'%f ',w1(n,:));
  %Store bias for neuron n.
  fprintf(fid,'%f\n',b1(n));
end
fclose(fid);

%Store layer two.
fid=fopen('anom001l2.txt','w');
%Number of neurons and number of inputs.
fprintf(fid,'%d %d\n',r2,c2);
%Cycle over neurons.
for n=1:r2
  %Store weights for neuron n.
  fprintf(fid,'%f ',w2(n,:));
  %Store bias for neuron n.
  fprintf(fid,'%f\n',b2(n));
end
fclose(fid);
