package saaps.ann;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.Serializable;

/**
The class NormParam contain the normalization parameters
needed for the preprocessing of a TimeSeries object before
it is input to a network. The parameters are
<ul>
<li> minIn, minimum value in the input data,
<li> maxIn, maximim value in the input data,
<li> minOut, minimum value into the network,
<li> maxOut, maximum value into the network,
</ul>
Thus the range [minIn,maxIn] is mapped to the range [minOut,maxOut].

@author Peter Wintoft 2000-11-08
@version 1.0
*/
public class NormParam implements Serializable
{
  private int numOfFields;
  private double minValueIn[];
  private double maxValueIn[];
  private double minValueOut[];
  private double maxValueOut[];

  /**
  Create a NormParam object from a saved object.
  <p>
  Note that this constructor assumes that all inputs
  are normalized with the same values.

  @param s file name
  */
  public NormParam(String s, int nof)
  {
    numOfFields = nof;
    RandomAccessFile raf;
    double minIn;
    double maxIn;
    double minOut;
    double maxOut;
    minValueIn = new double[numOfFields];
    maxValueIn = new double[numOfFields];
    minValueOut = new double[numOfFields];
    maxValueOut = new double[numOfFields];
    try
    {
      raf = new RandomAccessFile("/space/saaps/Models/" + s,"r");
      minIn = raf.readDouble();
      maxIn = raf.readDouble();
      minOut = raf.readDouble();
      maxOut = raf.readDouble();
      raf.close();
      for (int i=0;i<numOfFields;i++)
      {
        minValueIn[i] = minIn;
        maxValueIn[i] = maxIn;
        minValueOut[i] = minOut;
        maxValueOut[i] = maxOut;
      }
    }
    catch (IOException e)
    {
      System.out.println(e);
    }
  }

  public NormParam(double[] minIn, double[] maxIn, double[] minOut, double[] maxOut)
  {
    numOfFields = minIn.length;
    minValueIn = minIn;
    maxValueIn = maxIn;
    minValueOut = minOut;
    maxValueOut = maxOut;
  }

  /**
  Get the minIn value.
  */
  public double[] getMinValueIn()
  {
    return minValueIn;
  }

  /**
  Set the minIn value.
  */
  public void setMinValueIn(double[] m)
  {
    minValueIn = m;
  }

  /**
  Get the maxIn value.
  */
  public double[] getMaxValueIn()
  {
    return maxValueIn;
  }

  /**
  Set the maxIn value.
  */
  public void setMaxValueIn(double[] m)
  {
    maxValueIn = m;
  }

  /**
  Get the minOut value.
  */
  public double[] getMinValueOut()
  {
    return minValueOut;
  }

  /**
  Set the minOut value.
  */
  public void setMinValueOut(double[] m)
  {
    minValueOut = m;
  }

  /**
  Get the maxOut value.
  */
  public double[] getMaxValueOut()
  {
    return maxValueOut;
  }

  /**
  Set the maxOut value.
  */
  public void setMaxValueOut(double[] m)
  {
    maxValueOut = m;
  }

  /**
  Get the number of inputs.
  */
  public int getNumOfFields()
  {
    return numOfFields;
  }
}
