package saaps.dbt;

import java.text.DateFormat;
import java.text.NumberFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import java.util.Vector;

import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
* A class for ACE magnetometer objects.
*
* The class contains the date, Bx, By, Bz in GSE,
* and Bt from the ACE spacecraft. 
*
* There are methods to read
* and write date and ACE data.
*
* @author Peter Wintoft
* @version 1.0 2000-09-04
*/
public class ACEmagHourly extends DatabaseObject implements Serializable
{
  // Fields
  private static int numberOfIndexes = 4;
  private Date d;
  private double bx, by, bz, bt;
  
  // Constructors
  public ACEmagHourly()
  {
  }

  /**
   * This constructor is used if you want an "empty" object with this date.
   */
  public ACEmagHourly(Date D) {
    d = D;
    bx = Double.NaN;
    by = Double.NaN;
    bz = Double.NaN;
    bt = Double.NaN;
  }

  public ACEmagHourly(Date D,double[] data) {
    d = D;
    bx = data[0];
    by = data[1];
    bz = data[2];
    bt = data[3];
  }

  public ACEmagHourly(Date D,double BX,double BY,double BZ,double BT)
  {
    d = D;
    bx = BX;
    by = BY;
    bz = BZ;
    bt = BT;
  }
  public ACEmagHourly(long L,double BX,double BY,double BZ,double BT)
  {
    this(new Date(L),BX,BY,BZ,BT);
  }
  
  // Methods
  public double getBx()
  {
    return bx;
  }
  public double getBy()
  {
    return by;
  }
  public double getBz()
  {
    return bz;
  }
  public double getBt()
  {
    return bt;
  }

  //Implemented for DatabaseObject
  public Date getDate()
  {
    return d;
  }

  // This method is the generic form of accessing data.
  public Vector getData()
  {
  	Vector vec = new Vector(numberOfIndexes);
  	vec.addElement(new Double(bx));
  	vec.addElement(new Double(by));
  	vec.addElement(new Double(bz));
  	vec.addElement(new Double(bt));
  	return vec;
  }
  
  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  
    dos.writeLong(d.getTime());
    dos.writeDouble(bx);
    dos.writeDouble(by);
    dos.writeDouble(bz);
    dos.writeDouble(bt);
    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    d = D;
  }
  public void setDate(long L)
  {
    d = new Date(L);
  }
  public void setBx(double BX)
  {
    bx = BX;
  }
  public void setBy(double BY)
  {
    by = BY;
  }
  public void setBz(double BZ)
  {
    bz = BZ;
  }
  public void setBt(double BT)
  {
    bt = BT;
  }

  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }
     
  //Implemented for DatabaseObject 
  public double getData(int index) {
    //Date is not considered a data value
    switch (index) {
    case 0 :
      return(bx);
    case 1 :
      return(by);
    case 2 :
      return(bz);
    case 3 :
      return(bt);
    default :
      System.out.println("Index out of bounds for ACE");
      return (0.0);
    }
  }

  public String toString() {
//     // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE")); //en UK
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(bx) + " " + Double.toString(by) + " " + Double.toString(bz) + " " + Double.toString(bt));
//    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(getData()) );
    
  }

}
