package saaps.dbt;

import java.text.DateFormat;
import java.text.NumberFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import java.util.Vector;

/**
* A class for ACE objects.
*
* The class contains
* the date, Bx, By, Bz, n, and V from the ACE spacecraft.
* There are methods to read
* and write date and ACE data.
*
* @author Peter Wintoft
* @version 1.0 990913
*/
public class ACEswe extends DatabaseObject implements Serializable
{
  // Fields
  private static int numberOfIndexes = 2;
  private Date d;
  private double n, v;
  
  // Constructors
  public ACEswe() {

  }

  /**
   * This constructor is used if you want an "empty" object with this date.
   */
  public ACEswe(Date D) {
    d = D;
    n = Double.NaN;
    v = Double.NaN;
  }

  public ACEswe(Date D, double N, double V) {
    d = D;
    n  = N;
    v  = V;
  }
  public ACEswe(long L, double N, double V)
  {
    this(new Date(L),N,V);
  }

  public ACEswe(Date D, double[] data) {
    this(D,data[0],data[1]);
  }
  
  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }

  // Implemented for DatabaseObject 
  public Date getDate()
  {
    return d;
  }
  public double getN()
  {
    return n;
  }
  public double getV()
  {
    return v;
  }
  // This method is the generic form of accessing data.
  public Vector getData()
  {
  	Vector vec = new Vector(2);
  	vec.addElement(new Double(n));
  	vec.addElement(new Double(v));
  	return vec;
  }
     
  //Implemented for DatabaseObject 
  public double getData(int index) {
    //Date is not considered a data value
    switch (index) {
    case 0 :
      return(n);
    case 1 :
      return(v);
    default :
      System.out.println("Index out of bounds for ACE");
      return (0.0);
    }
  }

  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  
    dos.writeLong(d.getTime());
    dos.writeDouble(n);
    dos.writeDouble(v);
    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    d = D;
  }
  public void setDate(long L)
  {
    d = new Date(L);
  }
  public void setN(double N)
  {
    n = N;
  }
  public void setV(double V)
  {
    v = V;
  }

  public String toString() {
//     // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE")); //en UK
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " + Double.toString(n) + " " + Double.toString(v));
    
  }

}
