package saaps.dbt;

import java.io.*;
import java.util.*;

/**
 * ACEDB extends the Database class. It contains
 * the read and write methods that are used in
 * Database. The field is a vector of ACE objects.
 *
 *@author Peter Wintoft
 *@version 1.0 990913
 */
public class ACEsweDB extends Database {

  private static final double dataGapValueDefinition = -9999.9;

  // Static Fields used to initiate the MetaData
  private static String name="ACEsweDB";
  private static String description="ACE plasma data";
  private static String longDescription="The ACE plasm data consist of " +
    "1 minute resolution solar wind plasma data. " +
    "The data include solar wind density and velocity.";
  private static Calendar st=Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
  private static Calendar et=Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
  private static boolean ct=true;
  private static long step=60000; // One minute time step
  private static String[] fieldNames = { "Density" , "Speed" };
  private static String[] fieldUnits = { "cm^-3" , "km/s" };
  private static String[] fieldTypes = { "Double" , "Double" };

  // Fields.
  private ACEswe ace;
  private Vector acev;

  // Constructors.
  public ACEsweDB() {
    
    dataGapValue=dataGapValueDefinition;
  }
  public ACEsweDB(int i) {
    
    dataGapValue=dataGapValueDefinition;
    //    acev = ACEV;
    //store(acev);
  }
  /**
   * When a new class is added to the database this
   * constructor should be used which includes the
   * information that goes into the meta data file.
   *
   * The arguments are: Vector of ACE objects, the class name,
   * a short description, continuos time series (false/true), and
   * if a continous time series the time step.
   */
  public ACEsweDB(Vector ACEV) throws IOException {
    //, String des, boolean ct, long step) {
    //
    
    dataGapValue=dataGapValueDefinition;

    acev = ACEV;
    raf = new RandomAccessFile(baseDir + "ACEsweDB.data","rw");

    // Then get its metadata
    try { // To load the meta data for this database from file
      meta = new MetaData("ACEsweDB");
      isSet=true;
    }
    catch (IOException e) {
      System.err.println(e);
      System.out.println("Here we should handle the situation when no DB exists");
      try {
  meta = setDefaultMeta(((DatabaseObject) (acev.elementAt(0))).getDate(),((DatabaseObject) (acev.elementAt(acev.size()-1))).getDate()); //Takes static fields from XDB

  isSet=true;
      }
      catch (IOException ee) {
  System.err.println(ee);
  ee.printStackTrace();
      }
    }
    catch (ClassNotFoundException e) {
      System.err.println(e);
      e.printStackTrace();
    }

    store((Vector) acev); //Need to type cast so that store(Object obj) isn't used
  }

  // Methods.
  public Object read() throws IOException {
    ace = new ACEswe(raf.readLong(),raf.readDouble(),raf.readDouble());
    return (Object) ace;
  }

  public void myHello() {
    System.out.println("My ACEsweDB Hello");
  }

  public void write(Object obj) throws IOException {
    ace = (ACEswe) obj;
    raf.writeLong(ace.getDate().getTime());
    raf.writeDouble(ace.getN());
    raf.writeDouble(ace.getV());
  }
  // Implemented for Database abstract getDate
  public Date getDate(Object obj) {
    return ((ACEswe) obj).getDate();
  }

  /** This method is implemented for Database abstract metod.
   * It retrieves the latest data from the dataprovider and returns the vector of Database objects.
   */
  public Vector getLatestDatabaseObjects() {
    Vector vec = new Vector(8*30);
    Date mdate = new Date();

    ACEswe ace = new ACEswe(mdate,1.0,2.0);

    for(int i=0;i<8*30;i++) {
      vec.addElement(ace);
      mdate.setDate( (int) (mdate.getDate() + 10));
      ace.setDate(mdate);
      ace.setN(i);
      ace.setV(i);
    }
    vec.trimToSize();
    return vec;
  }

  /** Creates a metadata file for this class with the default values set 
   *to the static values defined above. OBS!, check calendar ....
   */
  protected MetaData setDefaultMeta(Date sd, Date ed) throws IOException {
    st.setTime(sd);
    st.clear(st.MILLISECOND);
    et.setTime(ed);
    et.clear(et.MILLISECOND);
    
    MetaData m = new MetaData(name,description, longDescription, st, et, ct, step,fieldNames,fieldUnits,fieldTypes);
    return m;
  }

  /**
   * Returns an "empty" metadata object. All fields but Start and end time
   * has valid values (as set in the respective database class).
   * The times will be set to that what is supplied.
   * OBS! This is not stored in the MetaData file !!
   */
  protected MetaData getEmptyMetaData(Date sd, Date ed) {
    MetaData m = new MetaData();
    st.clear(st.MILLISECOND);
    st.setTime(sd);
    et.clear(et.MILLISECOND);
    et.setTime(ed);
    m.setMetaData(name,description, longDescription,  st, et, ct, step, fieldNames, fieldUnits, fieldTypes );
    return m;
  }

}
