package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* BuildLANL reads the LANL electron data for certain energy ranges from the CDROMs.
* The data are then stored into the SAAPS database.
*@author Peter Wintoft
*@version 1.0 2001-02-13
*/
public class BuildLANL extends Object{
  //The directory.
//  private String dir = "/cdrom/cdrom0/";
  //Index to fields to read.
  private int[] fields = {1,2,3,4,7,12,20,29,36,42,49};
  //Bolean array indicating which fields to read.
//  private boolean[] useField = new boolean[117];
  //Field index.
//  private int fieldIndex = -1;
  //Maximum field index.
//  private int maxFieldIndex = 42;
  //The index to the data array.
//  private int dataIndex = 0;
  //Data gap value.
//  private static double dataGapValues = -999.0;
//    //F      B      Bx     ByGSE  BzGSE  ByGSM  BzGSM
//    { 999.9, 999.9, 999.9, 999.9, 999.9, 999.9, 999.9,
//    //sigmaF sigmaB sigmaBx sigmaBy sigmaBz
//      999.9, 999.9, 999.9,  999.9,  999.9,
//    //T          N      V       sigmaT     sigmaN sigmaV
//      9999990.0, 999.9, 9999.0, 9999990.0, 999.9, 9999.0,
//    //Dst    P1        P2        P4       P10      P30      P60
//      99999, 99999.90, 99999.90, 9999.99, 9999.99, 9999.99, 9999.99};
  //The stream tokenizer.
//  private StreamTokenizer streamTokenizer;
  //Date of one observation.
//  private Date date;
  //Data for one observation.
//  private double[] data = new double[fields.length];
  //A calendar object to do time calculations.
  private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
  //One day of LANL data.
  private LANLOneDay lanlOneDay;
  //The 24 LANL objects for one day.
  private Vector lanlVector = new Vector(24);
  
  private String lanlDir = null;
  
  public BuildLANL()
  {
    //Clear all time fields in the calendar object.
    cal.clear();
    //Set up useField.
////    useField = new boolean[43];
//    for (int i=0;i<useField.length;i++)
//      useField[i] = false;
//    for (int i=0;i<fields.length;i++)
//      useField[fields[i]-1] = true;
  }

  private void setDate(String[] dateStr)
  {
    int y=0;
    int m=0;
    int d=0;
    try {
      y = Integer.parseInt(dateStr[0]);
      m = Integer.parseInt(dateStr[1]);
      d = Integer.parseInt(dateStr[2]);
    }
    catch (NumberFormatException e) {
      System.err.println(e);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      System.err.println(e);
      System.err.println("Three arguments must be given:");
      System.err.println("   BuildLANL <year> <month> <date>");
      System.exit(0);
    }
    cal.set(cal.YEAR,y);
    cal.set(cal.MONTH,m-1);
    cal.set(cal.DATE,d);
  }
  
  private void readOneDayData() throws FileNotFoundException, IOException {
    System.out.println("Reading " + getFileName() + "  ...");
    lanlOneDay = new LANLOneDay(getFileName());
    lanlOneDay.skipHeader();
    lanlOneDay.readData();
  }
  
  private void hourlyAverages() {
    //An array to hold the averages. The length of the array is
    //the length of the fields array minus one. The first field is the UT field
    //and is not used for the averages.
    double[][] average = new double[24][fields.length-1];
    //Number of data points used for each field.
    int[][] nPoints = new int[24][fields.length-1];
    //Clear the vector of LANL objects.
    lanlVector.clear();
    //lanlVector = new Vector(24);
    //Go through the one day LANL data.
//    System.out.println(lanlOneDay.data.length);
    for (int i=0;i<lanlOneDay.data.length;i++) {
      //Get the time.
      int ut = (int) (lanlOneDay.data[i][0]);
//      System.out.println("i=" + i + " ut=" + ut);
      //Go through the fields.
      for (int j=0;j<(fields.length-1);j++) {
        //Check that it is not a datagap.
        if (lanlOneDay.data[i][fields[j+1]-1] > -999.0) {
          average[ut][j] += lanlOneDay.data[i][fields[j+1]-1];
          nPoints[ut][j]++;
        }
      }
    }
    //Create the vector of LANL objects.
//    System.out.print("Calculating average for UT=");
    for (int ut=0;ut<average.length;ut++) {
//      System.out.print(ut + ", ");
      //Normalize the averages.
      for (int k=0;k<average[ut].length;k++) {
        if (nPoints[ut][k]>0) {
          average[ut][k] /= nPoints[ut][k];
        }
        else {
          average[ut][k] = Double.NaN;
        }
      }
      //Create a LANL object.
      cal.set(cal.HOUR_OF_DAY,ut);
      LANL1990_095 lanlObj = new LANL1990_095(cal.getTime(),average[ut]);
      //Put into vector.
      lanlVector.add(lanlObj);
    }
    this.lanlVector.trimToSize();
//    System.out.println();
  }
  
  private void store() throws IllegalTimeSeriesException, IOException, Exception {
      System.out.println("Writing to database ...");
      if (!this.lanlVector.isEmpty()) {
        ObjectTimeSeries ts = new ObjectTimeSeries(this.lanlVector);
        Database db = Database.getInstance("LANL1990_095DB");
        db.store(ts);
        db.close();
      }
      else {
          System.out.println("Store failed! No data to store.");
      }
  }
  
  private void addOneDay() {
    cal.add(cal.DATE,1);
  }
  
  private String getFileName() {
      StringBuffer buf = lanlDir.equals("")?new StringBuffer("/cdrom/cdrom0/"):new StringBuffer(this.lanlDir);
      int year = cal.get(cal.YEAR) - 1900;
      int month = cal.get(cal.MONTH) + 1;
    int day = cal.get(cal.DATE);
    if (year<10) buf.append("0" + year);
    else buf.append(year);
    if (month<10) buf.append("0" + month);
    else buf.append(month);
    if (day<10) buf.append("0" + day);
    else buf.append(day);
    buf.append("_1990-095.fluxmerge");
    return buf.toString();
  }
  
  public String toString() {
    StringBuffer buf = new StringBuffer();
    for (int i=0;i<lanlVector.size();i++) {
      LANL1990_095 lanl = (LANL1990_095) lanlVector.elementAt(i);
      buf.append(lanl.getDate() + " : ");
      for (int j=0;j<lanl.getNumberOfDataIndexes();j++)
        buf.append(lanl.getData(j) + " ");
      buf.append("\n");
    }
    return buf.toString();
  }
  
  /*
  Check if the last date has been read on the CD
  
  @return True if last date on CD has been read
  */
  public boolean endCD() {
    return (cal.get(cal.MONTH)==0 & cal.get(cal.DATE)==1) |
           (cal.get(cal.MONTH)==6 & cal.get(cal.DATE)==1);
  }
    
  /*
  A class that contain one day of high resolution LANL data.
  */
  class LANLOneDay {
    //The number of data fields.
    int maxFields = 117;
    //A buffered reader for reading the data.
    BufferedReader buf;
    //The data is stored in an array. The first field is the samples, and the second the data field.
    double[][] data;
    
    /*
    Create one day of LANL data from the specified file.
    
    @param  fileName  The name of the file.
    */
    LANLOneDay(String fileName) throws FileNotFoundException {
      buf = new BufferedReader(new FileReader(fileName));
    }
    
    /*
    Skip the header line.
    */
    void skipHeader() throws IOException {
      String tmp = buf.readLine();
    }
    
    /*
    Read the data.
    */
    void readData() throws IOException, NumberFormatException {
      String textLine;
      Vector dataVec = new Vector(24*60);
      Double[] dataLine;
      while ((textLine = buf.readLine()) != null) {
//        System.out.println(textLine.substring(0,40));
        dataLine = new Double[maxFields];
        StringTokenizer st = new StringTokenizer(textLine);
        int i=0;
        while (st.hasMoreTokens()) {
          String dataValue = st.nextToken();
          dataLine[i++] = new Double(dataValue);
        }
        dataVec.add(dataLine);
      }
      dataVec.trimToSize();
      data = new double[dataVec.size()][maxFields];
      for (int i=0;i<data.length;i++) {
        dataLine = (Double[]) dataVec.elementAt(i);
        for (int j=0;j<maxFields;j++)
          data[i][j] = dataLine[j].doubleValue();
      }
    }
    
    /*
    Get the data.
    
    @return Array of array of doubles containing all the LANL data.
    */
//    double[][] getData() {
//      return data;
//    }
    
    /*
    Return a string of the data.
    
    @return The string.
    */
    public String toString() {
      StringBuffer strBuf = new StringBuffer();
      for (int i=0;i<data.length;i++) {
        for (int j=0;j<data[i].length;j++)
          strBuf.append(data[i][j] + " ");
        strBuf.append("\n");
      }
      return strBuf.toString();
    }
  }

  public static void main(String[] args)
  {
    BuildLANL build = new BuildLANL();
    //Set the first day of data to read.
//    build.setDate(1996,1,1);

    /* First argument is lanlDir if there are more than 3 arguments ....
     */
    if(args.length > 3) {
        build.lanlDir = args[0];
        if(! build.lanlDir.endsWith("/"))
            build.lanlDir = new String(build.lanlDir+"/");
        String[] tmp = {args[1],args[2],args[3]};
        build.setDate(tmp);
    }
    else {
        build.lanlDir = "";
        build.setDate(args);
    }
    //Check if file exist, else ask for new CD or quit.
    boolean cont = true;
    while (cont) {
      try {
        //Get one day of LANL data.
        build.readOneDayData();
        //Create hourly averages of selected fields.
        build.hourlyAverages();
        //Print the data.
//        System.out.println(build);
        //Store into the SAAPS database.
        build.store();
        //Add one day -> next day to read.
        build.addOneDay();
      }
      catch (IllegalTimeSeriesException e) {
        System.err.println(e);
      }
      catch (FileNotFoundException e) {
        System.out.println(e.getMessage());
        if (build.endCD())
          cont = false;
          //Add one day -> next day to read.
          build.addOneDay();
 /*       System.out.print("Insert new CD? (y/n) : ");
        try {
          int key = System.in.read();
          System.out.println(key);
          if (key != 'y') cont = false;
        }
        catch (IOException ioe) {
          System.err.println(ioe);
        }*/
      }
      catch (IOException e) {
        System.err.println(e);
      }
      catch (Exception e) {
        System.err.println(e);
      }
    }
  }
}
            
