/**
* The <code>DatabaseObject<\code> class shall be inherited by all
* data classes that will be stored in the database.
*/
package saaps.dbt;

import java.lang.reflect.*;
import java.util.*;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;


public abstract class DatabaseObject {

  //No common data fields yet

  //Some abstract methods that help us get data etc. 
  //When we don't know which object we got back.

  /** This static method is used to get a DatabaseObject object with
   * constructor arguments. The object returned has this date
   * but the datavalues are set to "not valid data".
   */
  public static DatabaseObject getInstance(String name,Date D) {

    DatabaseObject theDbo=null;

    try {
      Class classObject = Class.forName("saaps.dbt." + name);
      //Try to get a constructor with argument Vector for this database
      //Note that Constructor is from java.lang.reflect
      Class[] theArgs = { Class.forName("java.util.Date") };
      Constructor constr = classObject.getConstructor(theArgs);

      Object[] objArgs = { D };
      theDbo = (DatabaseObject) constr.newInstance( objArgs );
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return theDbo;
  }

  /** This static method is used to get a DatabaseObject object with
   * constructor arguments. The object returned has this date
   * and the data is passed in an array to the real constructor.
   */
  public static DatabaseObject getInstance(String name,Date D,double[] data) {

    DatabaseObject theDbo=null;

    try {
      Class classObject = Class.forName("saaps.dbt." + name);
      //Try to get a constructor with argument Vector for this database
      //Note that Constructor is from java.lang.reflect
      Class[] theArgs = { Class.forName("java.util.Date"), data.getClass() };
      Constructor constr = classObject.getConstructor(theArgs);

      Object[] objArgs = { D , data };
      theDbo = (DatabaseObject) constr.newInstance( objArgs );
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    return theDbo;
  }


  /** getNumberOfDataIndexes will return the number of data "types" this 
   *  DatabaseObject has.
   */
  public abstract int  getNumberOfDataIndexes();

  /** This method will return the data value of the DatabaseObject
   *  at position "index" converted to a double.
   *  If datavalue index doesn't exist we should either throw an exception
   *  or return a value that doesn't represent a number.
   */
  public abstract double getData(int index);

  /** getDate will return the Date associated with this DatabaseObject
   *
   */
  public abstract Date getDate();

  /**
   * Returns the size of the data that is written to file for this 
   * object type.
   */
  public abstract long getDataSize() throws IOException;

  /**
   * This method will return the size of the DatabaseObject in bytes.
   */
  public long getObjectSize() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    ObjectOutputStream    oos  = new ObjectOutputStream(baos);  
    oos.writeObject(this);  
    oos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

}
