package saaps.dbt;

import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.*;

import saaps.util.*;

/**
 * Handles the export of data from the SAAPS database. It can export
 * data from all Database classes.
 * The exported data has the following structure:
 * <p>
 * YYYY MM DD HH MM SS VAL(1) VAL(2) ... VAL(N)
 * <p>
 * See also {@link saaps.dbt.ImportData}.
 * @author Leif Kalla (leif@irf.se)
 */
public class ExportLatest  {
    
    private Database dbRef;
    private String useThis;
    
    public ExportLatest(String cname) {
        
        useThis = cname;
        dbRef = Database.getInstance(useThis,true);
    }
    
    public void export(long millis, Writer aWriter) throws IOException {
        BufferedWriter writer = new BufferedWriter(aWriter);

        try {            
            ObjectTimeSeries ts = dbRef.retrieveLatest(millis,true);
            for(int i=0;i<ts.getLength();i++) {
                writer.write((ts.getDataObject(i)).toString() + "\n");
            }
        }
        catch (IOException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch (DatabaseException e) {
            System.out.println(e);
        }
        catch (MetaNotSetException e) {
            System.out.println(e);            
        }
        catch (TimestepException e) {
            System.out.println(e);            
        }
        catch (IllegalTimeSeriesException e) {
            System.out.println(e);            
        }
        writer.close();
    }
    
    static public void main(String[] args) {
        int days=0,hours=0,minutes=0;
        
        try {
            if(args.length<2) {
                System.out.println("Usage: java saaps.dbt.ExportLatest <Db class name> <days> [ <hours> [ <minutes> ] ]");
                System.exit(0);
            }
            
            String dbClass = args[0];
            if(args.length>1)
                days = Integer.parseInt(args[1]);
            if(args.length>2)
                hours = Integer.parseInt(args[2]);
            if(args.length>3)
                minutes = Integer.parseInt(args[3]);
            
            OutputStreamWriter ost = new OutputStreamWriter(System.out);
            ExportLatest dl = new ExportLatest(args[0]);
            dl.export((days*24*60*60 + hours*60*60 + minutes*60)*1000,ost);
            dl.dbRef.close();
        }
        catch (Exception e) {
            System.out.println(e);
        }
    }
}
