//
//  ODIDatabase.java
//
//
//  Created by Peter Wintoft on 5/26/09.
//  Copyright 2009 Swedish Institute of Space Physics. All rights reserved.
//

package saaps.dbt;

import java.sql.*;
import java.util.Date;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.Locale;
import java.util.Properties;
import java.util.Vector;

import saaps.util.TimeSeries;

public class ODIDatabase {
	private Connection con;

	private static long cdfEpoch1970 = 62167219200000L;

	public ODIDatabase() {
	    try {
		Class.forName("com.mysql.jdbc.Driver");
		String host = System.getenv("ODI_HOST");
		String user = System.getenv("ODI_USER_3");
		String pw = System.getenv("ODI_PW_3");
		String db = System.getenv("ODI_DB");
		String socket = System.getenv("ODI_SOCKET");
		String port = System.getenv("ODI_PORT");

		String url = "jdbc:mysql://"+host+":"+port+"/"+db;
		Properties p = new Properties();
		p.setProperty("user",System.getenv("ODI_USER_3"));
		p.setProperty("password",System.getenv("ODI_PW_3"));
		if (System.getenv("ODI_SOCKET") != null)
			p.setProperty("socket",System.getenv("ODI_SOCKET"));
		this.con = DriverManager.getConnection(url,p);
//		this.con = DriverManager.getConnection(url,user,pw);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	}
	
//	public String getMetaString(String dataset) {
//	}
	
	public Date getEndDate(String dataset) {
//System.out.println(dataset);
//System.out.println(System.currentTimeMillis());
//		ResultSet res = this.query("SELECT epoch_end FROM dataset WHERE name=\""+dataset+"\"");
		ResultSet res = this.query("SELECT epoch_end FROM dataset WHERE sedat_name=\""+dataset+"\"");
		Date d = new Date();
		try {
			if(res.next()) {
//System.out.println(res.getString("epoch_end"));
				try {
					long cdfEpoch = Long.parseLong(res.getString("epoch_end")) - cdfEpoch1970;
					d.setTime(cdfEpoch);
				}
				catch (Exception e) {
					System.err.println("getEndDate() can not get epoch_end");
//					e.printStackTrace();
				}
			}
			else
				System.err.println("getEndDate() error");
		}
		catch(SQLException e) {
			e.printStackTrace();
		}
		return d;
	}
	
	public String getUnit(String dataset, String variable) {
		String unit = null;
		String v = this.removeNRV(variable);
//System.out.println("v: " + v);
		String q;

		q = "SELECT variable_attribute.value FROM variable_attribute JOIN variable_metadata JOIN dataset";
		q += " WHERE dataset.sedat_name=\""+dataset+"\"";
		q += " AND variable_metadata.dataset_id=dataset.id";
		q += " AND variable_metadata.name=\""+v+"\"";
		q += " AND variable_attribute.variable_metadata_id=variable_metadata.id";
		q += " AND variable_attribute.name=\"UNITS\"";
//System.out.println(q);
/*		if (variable.equals("dataset_file_id"))
		    unit = "none";
		else {
*/			ResultSet res = this.query(q);
			try {
				if(res.next()) {
//					System.out.println("ok");
					unit = res.getString("value");
				}
				else
					System.out.println("No unit "+variable);
			}
			catch(SQLException e) {
				e.printStackTrace();
			}
//		}
		return unit;
	}

	public String[] getFieldNames(String item) {
		String datasetname = getDataSetName(item);
		String q = "SELECT * FROM "+datasetname+" WHERE cdf_epoch="+this.cdfEpoch1970;
//System.out.println(q);
		int n = -1;
		ResultSet res = this.query(q);
		ResultSetMetaData meta = null;
		try {
		    meta = res.getMetaData();
		    n = meta.getColumnCount();
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
		String[] columnName = new String[n-3];
		String[] listName = this.getListName(item, n-3);
		try {
			for(int i=4; i<n; i++) {
				columnName[i-4] = meta.getColumnName(i);
			}
		}
		catch (SQLException e) {
			e.printStackTrace();
		}

		return columnName;
	}

	public String[] getListNames(String item) {
		String datasetname = getDataSetName(item);
		String q = "SELECT * FROM "+datasetname+" WHERE cdf_epoch="+this.cdfEpoch1970;
//System.out.println(q);
		int n = -1;
		ResultSet res = this.query(q);
		ResultSetMetaData meta = null;
		try {
		    meta = res.getMetaData();
		    n = meta.getColumnCount();
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
		String[] columnName = new String[n-3];
		String[] listName = this.getListName(item, n-3);

		return listName;
	}

//	}
	
	public Vector listDataset() {
		Vector dataset = new Vector(10);
		ResultSet res = this.query("SELECT name,sedat_name FROM dataset");
		try {
			while(res.next()) {
				Vector t = new Vector(2);
				t.addElement(res.getString(1));
				t.addElement(res.getString(2));
				dataset.addElement(t);
			}
		}
		catch(SQLException e) {
			e.printStackTrace();
		}
		dataset.trimToSize();
		return dataset;
	}


	public ResultSet query(String q) {
		ResultSet res = null;
		try {
			Statement stmt = this.con.createStatement();
			res = stmt.executeQuery(q);
		}
		catch(SQLException e) {
			e.printStackTrace();
		}
		return res;
	}
	
/*	public Vector retrieve(String item, Date sd, Date ed) {
		long cdfSd = sd.getTime() + this.cdfEpoch1970;
		long cdfEd = sd.getTime() + this.cdfEpoch1970;
		String q = "SELECT * FROM "+item+" WHERE cdf_epoch>="+cdfSd+" AND cdf_epoch<="+cdfEd;
		ResultSet res = this.query(q);
	}*/
	
	public String removeNRV(String s) {
		if (s == null)
		    return s;
		int d, i = s.indexOf(" (");
		if (i == -1)
			return s;
		else {
//System.out.println("S: " + s  + " pos: " + i + " sub: " + s.substring(0,i));
			return s.substring(0,i);
		}
	}

	public String removeDimNum(String s) {
		int d, i = s.lastIndexOf('_');
		if (s == null)
		    return s;
//System.out.println("S: " + s  + " pos: " + i);
		if (i == -1)
			return s;
		else {
		    try {
			d = Integer.parseInt(s.substring(i+1));
		    }
		    catch (Exception e) {
//			e.printStackTrace();
			return s;
		    }

			return s.substring(0,i);
		}
	}

	public String getDataSetName(String sedatname) {
		String s = sedatname + " not found";
		String q = "SELECT name FROM dataset WHERE sedat_name=\""+sedatname+"\"";
		try {
			ResultSet res = this.query(q);
			if (res.next()) {
				s = res.getString(1);
			}
			else
				System.err.println(s);
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
		return s;
	}

	public String[] getListName(String item, int vsize) {
		String[] ln = new String[vsize];
		// Get the CDF names, dims and sizes for record variant variables except Epoch.
		String q = "SELECT variable_metadata.name,dims,sizes FROM variable_metadata";
		q += " LEFT JOIN dataset ON variable_metadata.dataset_id=dataset.id";
		q += " WHERE dataset.sedat_name=\""+item+"\"";
		q += " AND variable_metadata.record_variance='t'";
		q += " AND variable_metadata.name!=\"epoch\"";

		try {
			String svar, currln;
			int dim, size, i = 0;
			ResultSet resCdfVarName = this.query(q);
			while (resCdfVarName.next()) {
				dim = resCdfVarName.getInt(2);
				size = resCdfVarName.getInt(3);
//System.out.println("i: " + i + " " + resCdfVarName.getString(1) + "  " + resCdfVarName.getLong(2) + "  " + resCdfVarName.getLong(3));
				currln = resCdfVarName.getString(1);
				if (dim == 0)
				    ln[i++] = currln;
				else {
					q = "SELECT va.value FROM variable_attribute AS va LEFT JOIN variable_metadata AS vm";
					q += " ON va.variable_metadata_id=vm.id LEFT JOIN dataset AS ds ON vm.dataset_id=ds.id";
					q += " WHERE ds.sedat_name=\""+item+"\" AND vm.name=\""+currln+"\" AND va.name=\"labl_ptr_1\"";
					ResultSet resVar = this.query(q);
//System.out.println(q);
					if (resVar.next()) {
						svar = resVar.getString(1);
						q = "SELECT nrv.nrv_value FROM variable_nrv AS nrv LEFT JOIN variable_metadata AS vm ON";
						q += " nrv.variable_metadata_id=vm.id LEFT JOIN dataset AS ds ON vm.dataset_id=ds.id WHERE";
						q += " ds.sedat_name=\""+item+"\" and vm.name=\""+svar+"\"";
						ResultSet resLinkVar = this.query(q);
						if (resLinkVar.first()) {
							do {	// Not nice but works
								ln[i++] = currln+" (" + resLinkVar.getString(1).trim() + ")";
							} while (resLinkVar.next());
						}
						else {		// No linked varable names just number them
							for (int j=1; j<=size; j++)
								ln[i++] = currln+" (" + j + ")";
						}
					}
					else {
						for (int j=1; j<=size; j++)
							ln[i++] = currln+" (" + j + ")";
//						System.err.println("INFO: no DB result for labl_ptr_1. dim: " + dim + " " + currln);
					}
				}
			}
//for (int j = 0; j < i; j++)
//    System.out.println("ln" + j +" " + ln[j]);
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
		return ln;
	}

	public TimeSeries retrieveTS(String item, Date sd, Date ed) {
		return retrieveTS(item, sd, ed, true);
	}

	public TimeSeries retrieveTS(String item, Date sd, Date ed,  boolean nanfilter) {
		long cdfSd = sd.getTime() + this.cdfEpoch1970;
		long cdfEd = ed.getTime() + this.cdfEpoch1970;
		String datasetname = getDataSetName(item);
//		String q = "SELECT * FROM "+datasetpref+item+" WHERE cdf_epoch>="+cdfSd+" AND cdf_epoch<="+cdfEd;
		String q = "SELECT * FROM "+datasetname+" WHERE cdf_epoch>="+cdfSd+" AND cdf_epoch<="+cdfEd;
		q += " ORDER BY cdf_epoch";
//System.out.println(q);
		ResultSet res = this.query(q);
		TimeSeries ts = null;
		try {
			ResultSetMetaData meta = res.getMetaData();
			int n = meta.getColumnCount();
			String[] columnName = new String[n-3];
			String[] columnUnit = new String[n-3];
			String[] listName = this.getListName(item, n-3);
			for(int i=4; i<n; i++) {
				columnName[i-4] = meta.getColumnName(i);
//System.out.print("colname:" + columnName[i-4] + " meta:" + i );
//				columnUnit[i-4] = this.getUnit(item,columnName[i-4]);
				columnUnit[i-4] = this.getUnit(item,listName[i-4]);
//System.out.println(" unit:" + columnUnit[i-4] + "  " + listName[i-4]);
			}
			Vector v = new Vector(1000);
			Vector vTime = new Vector(1000);
			while (res.next()) {
				double[] row = new double[n-3];
				for(int i=4; i<n; i++) {
					if (columnName[i-4].indexOf("flag") >= 0)
						row[i-4] = -1;
					else
						row[i-4] = res.getDouble(i);
				}
				v.add(row);
				vTime.add(res.getLong(1));
			}
			v.trimToSize();
			vTime.trimToSize();
			int m = v.size();
			if (m > 0) {
				Date[] date = new Date[m];
				double[][] data = new double[m][n-3];
				for(int i=0;i<m;i++) {
					data[i] = (double[]) v.elementAt(i);
					long time = ((Long) vTime.elementAt(i)).longValue() - this.cdfEpoch1970;
					date[i] = new Date(time);
				}
				// Get fillval and change fillval data to NaN for plot
				if (nanfilter) {
//System.out.println("nanfilter");
					double fillvalue;
					String qf;
					for (int i = 0; i < n-4; i++) {
						qf = "SELECT variable_attribute.value FROM variable_attribute JOIN variable_metadata JOIN dataset";
						qf += " WHERE dataset.sedat_name=\""+item+"\"";
						qf += " AND variable_metadata.dataset_id=dataset.id";
						qf += " AND variable_metadata.name=\""+removeNRV(listName[i])+"\"";
						qf += " AND variable_attribute.variable_metadata_id=variable_metadata.id";
						qf += " AND variable_attribute.name=\"FILLVAL\"";
//System.out.println(qf);
						ResultSet resf = this.query(qf);
						try {
							if(resf.next()) {
								fillvalue = resf.getDouble(1);
//System.out.println(listName[i] + " fillv:" +fillvalue);
								for (int jj = 0; jj < m; jj++) {
//System.out.print(" ("+jj+")>"+ data[jj][i]);
									if (data[jj][i] == fillvalue) {
										data[jj][i]=(double)java.lang.Double.NaN;
//System.out.print("*");
									}
								}
//System.out.println("");
							}
							else
								System.out.println(i+" fillvalue error "+listName[i]);
						}
						catch(SQLException e) {
							e.printStackTrace();
						}
					}
				}
				ts = new TimeSeries(date,data,columnName,columnUnit,listName);
			}
			else {
				System.err.println("retrieveTS() no data found. " + sd);
				Date[] date = new Date[1];
				double[][] data = new double[1][n-3];
				ts = new TimeSeries(date,data,columnName,columnUnit,listName);
			}
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
		return ts;
	}
  /**
   * Retrieves a TimeSeries object containing the averaged data for the
   * input event list.
   * <p>
   * Every date/time in the <code>events</code> list is used as time
   * reference for data retrieval, the <code>start</code> and 
   * <code>end</code> parameters define the time window for which 
   * data is retrieved.
   * If data for any of these time windows is missing an IOException 
   * is thrown.
   * The data field indicated is averaged over the events.
   * A <code>TimeSeries</code> object 
   * containing the data field indicated, the number of datapoints 
   * used in the average and the standard deviation  is created from 
   * the averaged data.
   * @param fieldIndex  index of the data field
   * @param events array of event times
   * @param start event window starts here (normally negative [milliseconds])
   * @param end event window ends here (normally positiv [milliseconds])
   * @param withAdditionalInfo Not used yet
   * @return <code>TimeSeries</code> object with averaged 
   * <code>fieldName</code> data for every event.
   * @exception TimeSeriesIndexException when <code>fieldIndex</code>
   * is out of bounds
   * @exception IOException when data for any of the event times are missing
   */
    public TimeSeries retrieveSEA(String sedatdb, int fieldIndex,Date[] events,long start, long end,boolean withAdditionalInfo) {

	Calendar sd = Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
	Calendar ed = Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
	Calendar dpTime = Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));

    /* An IOException is thrown if start date is before start of data
     * in file and if start date is after end of data in file.
     * XXX here we should get an DataNotFoundException instead !! 
     */
    TimeSeries[] tsArr = new TimeSeries[events.length];
    for(int i=0;i<events.length;i++) {
      sd.setTime(new Date(events[i].getTime()+start));
      ed.setTime(new Date(events[i].getTime()+end));
      tsArr[i] = retrieveTS(sedatdb,sd.getTime(),ed.getTime());
    }
    /* assume same length for all TimeSeries */
    double[][] data = new double[((TimeSeries) tsArr[0]).getLength()][3];
    Date[] tmpTimes = new Date[((TimeSeries) tsArr[0]).getLength()];

    /* calculate average, no. data pts, std.dev. */
    /* first loop over the data points */
    for(int j=0;j<((TimeSeries) tsArr[0]).getLength();j++) {
	/* second loop over the events */
	for(int i=0;i<tsArr.length;i++) {
	    double tmp = ((TimeSeries) tsArr[i]).getData(j,fieldIndex);
	    /* Skip NaN:s */
	    if(!Double.isNaN(tmp)) {
		data[j][0]+=tmp;      //the data itself
		data[j][1]++;        // Add one to the counts
		data[j][2] += Math.pow(tmp,2);      //std.dev. container
	    }
 	}
    }
    for(int j=0;j<((TimeSeries) tsArr[0]).getLength();j++) {
	if (data[j][1]<1) { //All datapoints were NaN
	    data[j][0] =  Double.NaN;
	    data[j][2] =  Double.NaN;
	}
	else {
	    data[j][0]/=data[j][1];
	    data[j][2]=Math.sqrt((data[j][2] - (data[j][1]*Math.pow(data[j][0],2)))/(data[j][1]-1));
	}
    }

    /* Create the times */
    long step = (end-start)/(((TimeSeries) tsArr[0]).getLength()-1);
    long stepStart = -1*((end-start)/2);
    for(int i=0;i<((TimeSeries) tsArr[0]).getLength();i++) {
	tmpTimes[i]= new Date(stepStart);
	stepStart+=step;
    }
    String descr = "Descriptor sql query failed";
    String q = "SELECT id FROM dataset WHERE sedat_name=\""+sedatdb+"\"";
//System.out.println(q);
    try {
	ResultSet res = this.query(q);
	res.next();
	String id = res.getString("id");

	q = "SELECT value FROM dataset_metadata WHERE attribute=\"Descriptor\" AND dataset_id="+id;
//System.out.println(q);
	res = this.query(q);
	res.next();
	descr = res.getString("value");
    }
    catch (SQLException e) {
	e.printStackTrace();
    }
    
    String[] fieldName = new String[3];
    String[] fieldUnit = new String[3];
//    fieldName[0] = new String("Index " + fieldIndex + " of " +
//			      meta.getDescr());
    fieldName[0] = new String("Index " + fieldIndex + " of " + descr);
    fieldName[1] = new String("no. data points");
    fieldName[2] = new String("standard deviation");
    fieldUnit[0] = new String("AVE");
    fieldUnit[1] = new String("AVE");
    fieldUnit[2] = new String("AVE");

    TimeSeries ts = new TimeSeries(tmpTimes,data,fieldName,fieldUnit);
//    ts.setShortDescription("SEA field " + fieldIndex + " of " + meta.getDescr());
    ts.setShortDescription("SEA field " + fieldIndex + " of " + descr);
    return (ts);
  }
}
