package saaps.math;

import Jama.Matrix;

/**
   The Java Math class extends the basic mathematical functions
   from java.lang.Math to use Jama.Matrix objects.
   <P>
   The tanh functions is also included.

@author Peter Wintoft
@version 2000-01-18
*/

public class Math
{
  /**
  Elementwize sine of a Matrix object.
  @param A Matrix.
  @return Matrix of the same size as the input matrix.
  */
  public static Matrix sin(Matrix A)
  {
    int rows = A.getRowDimension();
    int cols = A.getColumnDimension();
    Matrix B = new Matrix(rows,cols);
    for (int r=0;r<rows;r++)
    {
      for (int c=0;c<cols;c++)
      {
        B.set(r,c,java.lang.Math.sin(A.get(r,c)));
      }
    }
    return B;
  }

  /**
  Elementwize cosine of a Matrix object.
  @param A Matrix.
  @return Matrix of the same size as the input matrix.
  */
  public static Matrix cos(Matrix A)
  {
    int rows = A.getRowDimension();
    int cols = A.getColumnDimension();
    Matrix B = new Matrix(rows,cols);
    for (int r=0;r<rows;r++)
    {
      for (int c=0;c<cols;c++)
      {
        B.set(r,c,java.lang.Math.cos(A.get(r,c)));
      }
    }
    return B;
  }

  /**
  Elementwize tangent of a Matrix object.
  @param A Matrix.
  @return Matrix of the same size as the input matrix.
  */
  public static Matrix tan(Matrix A)
  {
    int rows = A.getRowDimension();
    int cols = A.getColumnDimension();
    Matrix B = new Matrix(rows,cols);
    for (int r=0;r<rows;r++)
    {
      for (int c=0;c<cols;c++)
      {
        B.set(r,c,java.lang.Math.tan(A.get(r,c)));
      }
    }
    return B;
  }

  /**
  Elementwize tangent hyperbolicus of a Matrix object.
  @param A Matrix.
  @return Matrix of the same size as the input matrix.
  */
  public static Matrix tanh(Matrix A)
  {
    int rows = A.getRowDimension();
    int cols = A.getColumnDimension();
    Matrix B = new Matrix(rows,cols);
    for (int r=0;r<rows;r++)
    {
      for (int c=0;c<cols;c++)
      {
        B.set(r,c,2/(1+java.lang.Math.exp(-2*A.get(r,c)))-1);
      }
    }
    return B;
  }

  public static Matrix logsig(Matrix A)
  {
    int rows = A.getRowDimension();
    int cols = A.getColumnDimension();
    Matrix B = new Matrix(rows,cols);
    for (int r=0;r<rows;r++)
    {
      for (int c=0;c<cols;c++)
      {
        B.set(r,c,1/(1+java.lang.Math.exp(-A.get(r,c))));
      }
    }
    return B;
  }

  public static Matrix tansig(Matrix A)
  {
    return tanh(A);
  }
  
  public static double mean(double[] x) {
      double y = 0;
      for (int i=0;i<x.length;i++) {
          y += x[i];
      }
      return y/x.length;
  }
  
  public static double linCorr(double[] x,double[] y) {
      double r1 = 0;
      double r2 = 0;
      double r3 = 0;
      double xm = Math.mean(x);
      double ym = Math.mean(y);
      for(int i=0;i<x.length;i++) {
          double xa = x[i]-xm;
          double ya = y[i]-ym;
          r1 += xa * ya;
          r2 += xa * xa;
          r3 += ya * ya;
      }
      return r1 / java.lang.Math.sqrt(r2) / java.lang.Math.sqrt(r3);
  }

/*
  public static double linCorrVN(double[] x, double[] y) {
      double[] tmp = VisualNumerics.math.Statistics.linearFit(x,y);
      return tmp[1];
  }
*/
}
