/*
 * DataPlotterFrame.java
 *
 * Created on August 4, 2000, 3:02 PM
 */
 
package saaps.saam;

import saaps.util.TimeSeries;
import saaps.util.TimeSeriesPlot;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.PrintJob;
import java.util.Date;

/** 
* This windows shows the plot requested from the Data Plotter Applet. The window
* contains a menu from which the plot axes and lines can be modified.
*
* @author  Peter Wintoft
* @version 1.0 2000-08-08
* @see      DataPlotterAppletAWT
* @see      DataPlotterAxisDialog
* @see      TimeSeries
* @see      TimeSeriesPlot.
*/
public class DataPlotterFrame extends java.awt.Frame {

  /** Creates new form DataPlotterFrame */
  public DataPlotterFrame() {
    initComponents ();
    pack ();
  }

  /**
  * Plot all data fields in the <code>TimeSeries</code> object
  * as a function of calendar time.
  *
  * @param  ts The time series to be plotted.
  */
  public DataPlotterFrame(TimeSeries ts) {
    initComponents ();
    pack ();
    TimeSeriesPlot tsp = new TimeSeriesPlot(ts);
    tsp.plotData();
    showPlot(tsp);
    checkboxMenuItemXAxis.setState(tsp.getXLog());
    checkboxMenuItemYAxis.setState(tsp.getYLog());
  }

  /**
  * Plot selected data fields from a <code>TimeSeries</code> object as a function of
  * another field, which can be the time field or another data field.
  *
  * @param  ts  The time series object.
  * @param  xf  The field for the x-axis. This can be a data field (xf>=0) or
  *             <code>xf=TimeSeriesPlot.REL_TIME</code> or <code>xf=TimeSeriesPlot.CAL_TIME</code>.
  * @param  yfs The fields for the y-axis.
  * @param  li  True if lines should be used.
  * @param  po  True if points should be used.
  * @param  xLabel  The x-label.
  * @param  yLabels The y-labels.
  * @param  xLog  True if the x-axis should be logarithmic.
  * @param  yLog  True if the y-axis should be logarithmic.
  */
  public DataPlotterFrame(TimeSeries ts, int xf, int[] yfs, boolean li, boolean po, String xLabel, String[] yLabels, boolean xLog, boolean yLog) {
    initComponents ();
    pack ();
    TimeSeriesPlot tsp = new TimeSeriesPlot(ts,xf,yfs,li,po,xLabel,yLabels,xLog,yLog);
    tsp.plotData();
    showPlot(tsp);
    checkboxMenuItemXAxis.setState(tsp.getXLog());
    checkboxMenuItemYAxis.setState(tsp.getYLog());
//    show();
//    checkboxMenuItemLines.setState(tsp.lines);
//    checkboxMenuItemPoints.setState(tsp.points);
  }
  
  public DataPlotterFrame(TimeSeries ts, int xf, int[] yfs, boolean li, boolean po, boolean xLog, boolean yLog)
  {
    initComponents();
    pack();
    TimeSeriesPlot tsp = new TimeSeriesPlot(ts,xf,yfs,li,po,xLog,yLog);
    tsp.plotData();
    showPlot(tsp);
    checkboxMenuItemXAxis.setState(tsp.getXLog());
    checkboxMenuItemYAxis.setState(tsp.getYLog());
  }

  private void showPlot(TimeSeriesPlot plot)
  {
//    if (panel.getComponentCount() > 0)
//      panel.remove(0);
    panel.add(plot,0);
//    panel.setSize(plot.getSize());
//    panel.validate();
    plot.repaint();
    pack();
  }


  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    menuBar = new java.awt.MenuBar ();
    menuFile = new java.awt.Menu ();
    menuItemPrint = new java.awt.MenuItem ();
    menuItemQuit = new java.awt.MenuItem ();
    menuAxis = new java.awt.Menu ();
    checkboxMenuItemXAxis = new java.awt.CheckboxMenuItem ();
    checkboxMenuItemYAxis = new java.awt.CheckboxMenuItem ();
    menuItemAxisFill = new java.awt.MenuItem ();
    menuItemSetAxesLimits = new java.awt.MenuItem ();
    menuItemAxisCopy = new java.awt.MenuItem ();
    menuItemAxisPaste = new java.awt.MenuItem ();
    menuLines = new java.awt.Menu ();
    checkboxMenuItemLines = new java.awt.CheckboxMenuItem ();
    checkboxMenuItemPoints = new java.awt.CheckboxMenuItem ();
    panel = new java.awt.Panel ();

      menuFile.setLabel ("File");
      menuFile.setName ("menuFile");
  
        menuItemPrint.setEnabled (false);
        menuItemPrint.setLabel ("Print ...");
        menuItemPrint.setName ("menuitemPrint");
        menuItemPrint.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            menuItemPrintActionPerformed (evt);
          }
        }
        );
    
        menuFile.add (menuItemPrint);
    
        menuFile.addSeparator ();
        menuItemQuit.setLabel ("Quit");
        menuItemQuit.setName ("menuitemQuit");
        menuItemQuit.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            menuItemQuitActionPerformed (evt);
          }
        }
        );
    
        menuFile.add (menuItemQuit);
      menuBar.add (menuFile);
      menuAxis.setLabel ("Axis");
      menuAxis.setName ("menuAxis");
  
        checkboxMenuItemXAxis.setLabel ("X Log");
        checkboxMenuItemXAxis.setName ("chkmenuitemXAxis");
        checkboxMenuItemXAxis.addItemListener (new java.awt.event.ItemListener () {
          public void itemStateChanged (java.awt.event.ItemEvent evt) {
            checkboxMenuItemXAxisItemStateChanged (evt);
          }
        }
        );
    
        menuAxis.add (checkboxMenuItemXAxis);
        checkboxMenuItemYAxis.setLabel ("Y Log");
        checkboxMenuItemYAxis.setName ("chkmenuitemYAxis");
        checkboxMenuItemYAxis.addItemListener (new java.awt.event.ItemListener () {
          public void itemStateChanged (java.awt.event.ItemEvent evt) {
            checkboxMenuItemYAxisItemStateChanged (evt);
          }
        }
        );
    
        menuAxis.add (checkboxMenuItemYAxis);
    
        menuAxis.addSeparator ();
        menuItemAxisFill.setLabel ("Fill");
        menuItemAxisFill.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            menuItemAxisFillActionPerformed (evt);
          }
        }
        );
    
        menuAxis.add (menuItemAxisFill);
        menuItemSetAxesLimits.setLabel ("Set Axes Limits ...");
        menuItemSetAxesLimits.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            menuItemSetAxesLimitsActionPerformed (evt);
          }
        }
        );
    
        menuAxis.add (menuItemSetAxesLimits);
    
        menuAxis.addSeparator ();
        menuItemAxisCopy.setEnabled (false);
        menuItemAxisCopy.setLabel ("Copy");
    
        menuAxis.add (menuItemAxisCopy);
        menuItemAxisPaste.setEnabled (false);
        menuItemAxisPaste.setLabel ("Paste");
    
        menuAxis.add (menuItemAxisPaste);
      menuBar.add (menuAxis);
      menuLines.setLabel ("Lines");
      menuLines.setName ("menuLines");
  
        checkboxMenuItemLines.setState (true);
        checkboxMenuItemLines.setLabel ("Lines");
        checkboxMenuItemLines.setName ("chkmenuitemLines");
        checkboxMenuItemLines.addItemListener (new java.awt.event.ItemListener () {
          public void itemStateChanged (java.awt.event.ItemEvent evt) {
            checkboxMenuItemLinesItemStateChanged (evt);
          }
        }
        );
    
        menuLines.add (checkboxMenuItemLines);
        checkboxMenuItemPoints.setLabel ("Points");
        checkboxMenuItemPoints.setName ("chkmenuitemPoints");
        checkboxMenuItemPoints.addItemListener (new java.awt.event.ItemListener () {
          public void itemStateChanged (java.awt.event.ItemEvent evt) {
            checkboxMenuItemPointsItemStateChanged (evt);
          }
        }
        );
    
        menuLines.add (checkboxMenuItemPoints);
      menuBar.add (menuLines);
    setLayout (new java.awt.FlowLayout ());
    addComponentListener (new java.awt.event.ComponentAdapter () {
      public void componentResized (java.awt.event.ComponentEvent evt) {
        formComponentResized (evt);
      }
    }
    );
    addWindowListener (new java.awt.event.WindowAdapter () {
      public void windowClosing (java.awt.event.WindowEvent evt) {
        exitForm (evt);
      }
    }
    );

    panel.setBackground (new java.awt.Color (204, 204, 204));
    panel.setForeground (java.awt.Color.black);
    panel.setName ("panel3");
    panel.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (panel);

    setMenuBar (menuBar);

  }//GEN-END:initComponents

  private void menuItemSetAxesLimitsActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemSetAxesLimitsActionPerformed
// Add your handling code here:
    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    double[] xRange = tsp.getXRange();
    double[] yRange = tsp.getYRange();
    System.out.println(xRange[0] + " " + xRange[1] + " " + yRange[0] + " " + yRange[1]);
    DataPlotterAxisDialog dialog = new DataPlotterAxisDialog(this,true,xRange,yRange);
    dialog.show();
    xRange = dialog.getXRange();
    yRange = dialog.getYRange();
    tsp.setXRange(xRange[0],xRange[1]);
    tsp.setYRange(yRange[0],yRange[1]);
    dialog.dispose();
    showPlot(tsp);
  }//GEN-LAST:event_menuItemSetAxesLimitsActionPerformed

  private void menuItemAxisFillActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemAxisFillActionPerformed
// Add your handling code here:
    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    tsp.fillPlot();
    showPlot(tsp);
  }//GEN-LAST:event_menuItemAxisFillActionPerformed

  private void formComponentResized (java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
// Add your handling code here:
//    System.out.println("Size changed!!");
    Dimension dim = getSize();
    Insets ins = getInsets();
//    System.out.println("this.getSize()=" + dim);
//    System.out.println("this.getInsets()=" + ins);
//    System.out.println("panel.getPrefferedSize()=" + panel.getPreferredSize());
//    panel.validate();
//    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
//    tsp.setSize(dim);
//    showPlot(tsp);
//    panel.setSize(dim);
//    System.out.println(panel.getSize());
    panel.validate();
//    System.out.println(panel.getSize());

//    pack();
  }//GEN-LAST:event_formComponentResized

  private void menuItemPrintActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemPrintActionPerformed
// Add your handling code here:
    PrintJob pj = getToolkit().getPrintJob(this,"Frame print",null);
    if (pj != null)
    {
      Graphics gr = pj.getGraphics();
      Image im = panel.createImage(400,400);
      boolean isDrawn = gr.drawImage(im, 0, 0, this);
      System.out.println("Image has been drawn: " + isDrawn);
      gr.dispose();
      pj.end();
    }
  }//GEN-LAST:event_menuItemPrintActionPerformed

  private void checkboxMenuItemXAxisItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemXAxisItemStateChanged
// Add your handling code here:
    //Get the state.
    boolean xLog = checkboxMenuItemXAxis.getState();
    //Get the plot.
    TimeSeriesPlot plot = (TimeSeriesPlot) panel.getComponent(0);
    //Set the scale.
    plot.setXLog(xLog);
    //Redraw.
    plot.plotData();
    showPlot(plot);

  }//GEN-LAST:event_checkboxMenuItemXAxisItemStateChanged

  private void checkboxMenuItemLinesItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemLinesItemStateChanged
// Add your handling code here:
    //Get the state.
    boolean lines = checkboxMenuItemLines.getState();
    boolean points = checkboxMenuItemPoints.getState();
    //Points or lines must be set.
    if (!lines && !points)
    {
      points = true;
      checkboxMenuItemPoints.setState(points);
    }
    //Get the plot.
    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    //Set the state.
    tsp.setLineState(lines);
    tsp.setPointState(points);
//    System.out.println("l=" + tsp.getLineState() + " p=" + tsp.getPointState());
    //Redraw.
    tsp.plotData();
    showPlot(tsp);
  }//GEN-LAST:event_checkboxMenuItemLinesItemStateChanged

  private void checkboxMenuItemPointsItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemPointsItemStateChanged
// Add your handling code here:
    //Get the state.
    boolean lines = checkboxMenuItemLines.getState();
    boolean points = checkboxMenuItemPoints.getState();
    //Points or lines must be set.
    if (!lines && !points)
    {
      lines = true;
      checkboxMenuItemLines.setState(lines);
    }
    //Get the plot.
    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    //Set the state.
    tsp.setLineState(lines);
    tsp.setPointState(points);
//    System.out.println("l=" + tsp.getLineState() + " p=" + tsp.getPointState());
    //Redraw.
    tsp.plotData();
    showPlot(tsp);
  }//GEN-LAST:event_checkboxMenuItemPointsItemStateChanged

  private void checkboxMenuItemYAxisItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemYAxisItemStateChanged
// Add your handling code here:
    System.out.println("State changed");
    //Get the state.
    boolean yLog = checkboxMenuItemYAxis.getState();
    //Get the plot.
    TimeSeriesPlot plot = (TimeSeriesPlot) panel.getComponent(0);
    //Set the scale.
    plot.setYLog(yLog);
    //Redraw.
    plot.plotData();
    showPlot(plot);

  }//GEN-LAST:event_checkboxMenuItemYAxisItemStateChanged

  private void menuItemQuitActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemQuitActionPerformed
// Add your handling code here:
    dispose();
    System.exit(0);
  }//GEN-LAST:event_menuItemQuitActionPerformed

  /** Exit the Application */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
    dispose();
    System.exit (0);
  }//GEN-LAST:event_exitForm

  /**
  * The main method is used for testing.
  *
  * @param args the command line arguments
  */
  public static void main (String args[]) {
    //Test code for stand alone application.
    Date[] l = {new Date(0L), new Date(1000L), new Date(2000L), new Date(3000L), new Date(4000L)};
    double[] d = {0.0,1.0,4.0,9.0,16.0};
    TimeSeries ts = new TimeSeries(l,d);
    ts.setShortDescription("This is a test!");
    DataPlotterFrame dpf = new DataPlotterFrame(ts);
    dpf.show();
  }


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private java.awt.MenuBar menuBar;
  private java.awt.Menu menuFile;
  private java.awt.MenuItem menuItemPrint;
  private java.awt.MenuItem menuItemQuit;
  private java.awt.Menu menuAxis;
  private java.awt.CheckboxMenuItem checkboxMenuItemXAxis;
  private java.awt.CheckboxMenuItem checkboxMenuItemYAxis;
  private java.awt.MenuItem menuItemAxisFill;
  private java.awt.MenuItem menuItemSetAxesLimits;
  private java.awt.MenuItem menuItemAxisCopy;
  private java.awt.MenuItem menuItemAxisPaste;
  private java.awt.Menu menuLines;
  private java.awt.CheckboxMenuItem checkboxMenuItemLines;
  private java.awt.CheckboxMenuItem checkboxMenuItemPoints;
  private java.awt.Panel panel;
  // End of variables declaration//GEN-END:variables

}