package saaps.saam;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import ptolemy.plot.Plot;
import saaps.util.TimeSeries;

/**
*
* @author   Peter Wintoft
* @version  2000-02-18
*/
public class DataPlotterPlot extends Object
{

  private Plot plot;

  public DataPlotterPlot(TimeSeries ts, int xf, int[] yfs,
                         boolean lines, boolean points,
                         String xLabel, String[] yLabels,
                         boolean xLog, boolean yLog)
  {
    //Initialize plot.
    plot = new Plot();
    plot.setSize(400,300);
    plot.setButtons(true);

    //Get the fields.
    double[] x = new double[ts.getLength()];
    double[][] y = new double[ts.getLength()][];
    String timeUnit = "seconds";
    if (xf==-1)
    {
      Date[] d = ts.getDates();
      double x1 = (double) d[0].getTime()/1000;
      double x2 = (double) d[d.length-1].getTime()/1000;
      double xw = x2-x1;
      double timeDiv = 1000;
      if (xw>60 & xw<=3600)
      {
        timeDiv = 1000*60;
        timeUnit = "minutes";
      }
      if (xw>3600 & xw<=3600*24)
      {
        timeDiv = 1000*3600;
        timeUnit = "hours";
      }
      if (xw>3600*24)
      {
        timeDiv = 1000*3600*24;
        timeUnit = "days";
      }
      for (int i=0;i<d.length;i++)
        x[i] = (double) ((d[i].getTime() - d[0].getTime())) / timeDiv;
    }
    else
    {
      for (int i=0;i<ts.getLength();i++)
        x[i] = ts.getData(i,xf);
    }
    for (int i=0;i<ts.getLength();i++)
    {
      y[i] = new double[yfs.length];
      for (int j=0;j<yfs.length;j++)
        y[i][j] = ts.getData(i,yfs[j]);
    }

    //Log plot?
    plot.setXLog(xLog);
    plot.setYLog(yLog);

    //Plot the data.
    for (int j=0;j<y[0].length;j++)
    {
      boolean connect = false;
      for (int i=0;i<x.length;i++)
      {
        if (Double.isNaN(x[i]) | Double.isNaN(y[i][j]))
        {
          connect = false;
        }
        else
        {
          plot.addPoint(j,x[i],y[i][j],connect);
          connect = lines;
        }
      }
    }
    if (points)
      plot.setMarksStyle("various");

    //Set labels.
    if (xf==-1)
    {
      Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
//      cal.setTime(ts.getDate(0));
      DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,DateFormat.MEDIUM,new Locale("uk","UK"));
      df.setCalendar(cal);
      plot.setXLabel(timeUnit + " since " + df.format(ts.getDate(0)));
    }
    else
    {
      plot.setXLabel(xLabel);
    }
    for (int i=0;i<yLabels.length;i++)
      plot.addLegend(i,yLabels[i]);

    //Set the axes.
    //  int year = cal.get(Calendar.YEAR);
    //  int month = cal.get(Calendar.MONTH)+1;
    //  int day = cal.get(Calendar.DATE);
    //  plot.setXLabel("Days from " + year + "-" + month + "-" + day);
    //    plot.setXLabel("Days from " + cal.getTime() );
    //  plot.setYRange(-1,1);
  }

  public Plot getPlot()
  {
    return plot;
  }

}
