/*
 * ElFluence.java
 *
 * Created on June 27, 2001, 8:39 AM
 */

package saaps.saam;

import java.io.IOException;
import java.util.Date;

import saaps.dbt.Database;
import saaps.dbt.ODIDatabase;
import saaps.util.TimeSeries;

/**
 *
 * @author  root
 * @version 
 */
public class ElFluence extends Object {

    //The database.
//    private GOES08partDB db = (GOES08partDB) Database.getInstance("GOES08partDB");

    //Sedat database name and ODI database
//    private String sedatdbname = "GOES_?08_?";
    private String sedatdbname = "KPAP_3h";	// FAKE DATA
    
    private ODIDatabase odiDatabase = new ODIDatabase();
    
    //The fluencees. The first element is the 0.6 MeV fluence
    //and the second element is the 2.0 MeV fluence.
    private double[] fluence = new double[2];
    
    //The flux. There are 288 five-minute values per day.
    private double[][] flux = new double[2][288];
    
    //The number of flux values might be less than 288 if
    //there are NaN's in the data. This arrays contain the
    //number of flux values that is actually present.
    private int[] numOfFlux = new int[2];
    
    /** Creates new ElFluence */
    public ElFluence() {
    }
    
    ////////////////////////////
    // PUBLIC METHODS
    ////////////////////////////
    
    /**
     *Set the date, load the GOES-08 0.6 MeV and 2.0 MeV
     *electron fluxes, and calculate the one day fluence.
     *
     *@param d The date for which the fluence should be calculated.
     */
    public void setDate(Date d) throws IOException {
        //Read the flux.
        this.readFlux(d);
        //Calculate the fluence.
        this.calcFluence();
    }
    
    /**
     *Get the 0.6 MeV electron fluence. The unit is cm^-2 sr^-1.
     */
    public double get06Fluence() {
        return this.fluence[0];
    }
    
    /**
     *Get the 2.0 MeV electron fluence. The unit is cm^-2 sr^-1.
     */
    public double get20Fluence() {
        return this.fluence[1];
    }

    ///////////////////////////////
    // PRIVATE METHODS
    ///////////////////////////////
    
    /**
     *Read the flux.
     */
    private void readFlux(Date d1) throws IOException {
        //The flux.
        TimeSeries ts = null;
        //The end date, 24 hours after the start date.
        //24 hours = 24*60*60*1000 ms = 86400000 ms.
        Date d2 = new Date(d1.getTime() + 86400000L);
//        try {
        ts = odiDatabase.retrieveTS(sedatdbname,d1,d2,true);
/*        }
        catch (IOException e) {
            throw new IOException("Can't get data for period:\n\nstart :"+d1+
            "\n\nend   :"+d2);
        }
*/        this.numOfFlux[0] = 0;
        this.numOfFlux[1] = 0;
        //Get the smallest of the ts length and the flux length.
        int imax = this.flux[0].length;
        if (imax > ts.getLength()) imax = ts.getLength();
        //Extract the data.
        for (int i=0;i<imax;i++) {
/*            //0.6 MeV
            if (!Double.isNaN(ts.getData(i,6))) {
                this.numOfFlux[0]++;
                this.flux[0][i] = ts.getData(i,6);
            }
            //2.0 MeV
            if (!Double.isNaN(ts.getData(i,7))) {
                this.numOfFlux[1]++;
                this.flux[1][i] = ts.getData(i,7);
            }*/
            // Test for kpap_3h
            //0.6 MeV
            if (!Double.isNaN(ts.getData(i,0))) {
                this.numOfFlux[0]++;
                this.flux[0][i] = ts.getData(i,0);
System.out.println(".6>"+this.flux[0][i]);
            }
            //2.0 MeV
            if (!Double.isNaN(ts.getData(i,2))) {
                this.numOfFlux[1]++;
                this.flux[1][i] = ts.getData(i,2);
System.out.println("2>"+this.flux[1][i]);
            }
        }
    }
    
    private void calcFluence() {
        this.fluence[0] = 0;
        this.fluence[1] = 0;
        //Loop over the energy levels.
        for (int i=0;i<this.fluence.length;i++) {
            //Sum the flux data.
            for (int j=0;j<this.numOfFlux[i];j++) {
                this.fluence[i] += this.flux[i][j];
            }
            //Calculate the daily fluence.
            //There should be 288 five-minute values per day.
            //The flux is counted per second and there are
            //5*60=300 seconds in a five minute interval.
            if (this.numOfFlux[i]>0) {
                this.fluence[i] *= (300 * 288.0 / this.numOfFlux[i]);
            }
            else {
                this.fluence[i] = Double.NaN;
            }
        }
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        try {
            if (args.length<6) {
                System.out.println("Usage: ElFluence <year> <month> <date> <hour> <minute> <second>");
            }
            int[] date = new int[6];
            for (int i=0;i<date.length;i++) {
                date[i] = Integer.parseInt(args[i]);
            }
            date[1]--;
            java.util.TimeZone tz = java.util.TimeZone.getTimeZone("GMT");
            java.util.Calendar cal = java.util.Calendar.getInstance(tz);
            cal.set(java.util.Calendar.YEAR,date[0]);
            cal.set(java.util.Calendar.MONTH,date[1]);
            cal.set(java.util.Calendar.DATE,date[2]);
            cal.set(java.util.Calendar.HOUR_OF_DAY,date[3]);
            cal.set(java.util.Calendar.MINUTE,date[4]);
            cal.set(java.util.Calendar.SECOND,date[5]);
            cal.set(java.util.Calendar.MILLISECOND,0);
            ElFluence ef = new ElFluence();
            ef.setDate(cal.getTime());
            System.out.println(ef.numOfFlux[0] + " " + ef.numOfFlux[1]);
            System.out.println(ef.get06Fluence());
            System.out.println(ef.get20Fluence());
        }
        catch (IOException e) {
            System.out.println("Data access error");
            System.out.println(e);
        }
    }
}
