package saaps.saam;

import java.io.IOException;
import java.rmi.*;
import java.util.Date;

import saaps.dbt.*;
import saaps.math.*;
import saaps.util.*;

/**
*
* @author
* @version
*/
public class SEAData extends Thread
{
  //The uploaded time series.
  private TimeSeries sea;
  //The database connection.
  private DatabaseRMI dbRMI;
  //The event dates.
  private Date[] eventDates;
  //Start and end windows.
  private long startWindow, endWindow;
  //Name of the database object.
  private String dboName;
  //Field to use.
  private int field;
  //The data dialog.
  SEADataDialog seaDD;
  //The thread to this object.
//  Thread thread;
  //Flag if thread is running.
//  boolean isRunning;

  private java.awt.Frame parent;
  
  public SEAData(Date[] ed,long sw, long ew, DatabaseRMI dbr, String dbo, int f, SEADataDialog seadd)
  throws RemoteException, TimeSeriesIndexException, IllegalTimeSeriesException, IOException {
      eventDates = ed;
      startWindow = sw;
      endWindow = ew;
      dbRMI = dbr;
      dboName = dbo;
      field = f;
      seaDD = seadd;
      
      //If number of events is less than 4 then load each event individually,
      //else load only mean and standard deviation.
      if (eventDates.length<4)
          getAllTimeSeries();
      else
          getSEATimeSeries();
  }
  
  public SEAData(java.awt.Frame parent,Date[] ed,long sw, long ew, DatabaseRMI dbr, String dbo, int f) {
      eventDates = ed;
      startWindow = sw;
      endWindow = ew;
      dbRMI = dbr;
      dboName = dbo;
      field = f;
      this.parent=parent;
  }
  
  public void run() {
      DataLoadingDialog dld = new DataLoadingDialog(parent,false,this);
      dld.show();      
      try {
          //If number of events is less than 4 then load each event individually,
          //else load only mean and standard deviation.
          if (eventDates.length<4)
              getAllTimeSeries();
          else
              getSEATimeSeries();
          dld.dispose();

          PlotFrame seaPlot = new PlotFrame(getTimeSeries());
          seaPlot.useRelTime();
          seaPlot.plotData();
          seaPlot.show();
      }
      catch (RemoteException re) {
          dld.dispose();
          new ErrorDialog(parent,re.getMessage());
          System.err.println(re);
      }
      catch (TimeSeriesIndexException tsie) {
          dld.dispose();
          new ErrorDialog(parent,tsie.getMessage());
          System.err.println(tsie);
      }
      catch (IllegalTimeSeriesException itse) {
          dld.dispose();
          new ErrorDialog(parent,itse.getMessage());
          System.err.println(itse);
      }
      catch (IOException ioe) {
          dld.dispose();
          new ErrorDialog(parent,ioe.getMessage());
          System.err.println(ioe);
      }
  }
  
  private void getAllTimeSeries() throws RemoteException, IOException
  {
    //The individual time series objects.
    TimeSeries[] ts = new TimeSeries[eventDates.length];
    //Load the time series.
//    try
//    {
      for (int i=0;i<ts.length;i++)
      {
        long eventTime = eventDates[i].getTime();
        Date sd = new Date(eventTime + startWindow);
        Date ed = new Date(eventTime + endWindow);
        ts[i] = dbRMI.getTimeSeries(dboName,sd,ed);
      }
//    }
//    catch(RemoteException re)
//    {
//      System.err.println(re);
//    }
//    catch(IOException ioe)
//    {
//      System.err.println(ioe);
//    }
    //Create the SEA time series containing only the selected field.
    //Create the dates.
    Date[] dates = new Date[ts[0].getLength()];
    for (int row=0;row<dates.length;row++)
      dates[row] = new Date(ts[0].getDate(row).getTime()-eventDates[0].getTime());
    //Create the data.
    double[][] data = new double[ts[0].getLength()][eventDates.length];
    for (int row=0;row<data.length;row++)
      for(int col=0;col<data[0].length;col++)
        data[row][col] = ts[col].getData(row,field);
    //Create the field names and field units.
    String[] fn = new String[eventDates.length];
    String[] fu = new String[eventDates.length];
    for (int i=0;i<fn.length;i++)
    {
      int j=i+1;
      fn[i] = new String("Event " + j);
      String[] fus = ts[i].getFieldUnits();
      fu[i] = new String(fus[field]);
    }
    //Create the time series.
    sea = new TimeSeries(dates,data,fn,fu);
    String[] tsFieldNames = ts[0].getFieldNames();
    //Set short description.
    //This code is temporarily,
    sea.setShortDescription(dboName + " " + tsFieldNames[field]);
    //shall be replaced with this code.
//    sea.setShortDescription(ts[0].getShortDescription() + " " + tsFieldNames[field]);
  }
  
  private void getSEATimeSeries() throws RemoteException, TimeSeriesIndexException,
    IllegalTimeSeriesException, IOException
  {
    //The loaded SEA time series.
    TimeSeries ts = null;
//    try
//    {
      ts = dbRMI.retrieveSEA(dboName,field,eventDates,startWindow,endWindow,true);
//    }
/*    catch (RemoteException re)
    {
      System.err.println(re);
    }
    catch (TimeSeriesIndexException tsie)
    {
      System.err.println(tsie);
    }
    catch (IllegalTimeSeriesException itse)
    {
      System.err.println(itse);
    }
    catch (IOException ioe)
    {
      System.err.println(ioe);
    }*/
    //Create time series for plotting.
    //The fields are: mean, mean+std, mean-std.
    double[][] data = new double[ts.getLength()][3];
    for (int row=0;row<data.length;row++)
    {
      data[row][0] = ts.getData(row,0);
      data[row][1] = data[row][0] + ts.getData(row,1);
      data[row][2] = data[row][0] - ts.getData(row,1);
    }
    //Set field names and field units.
    String[] fn = {"Mean","Mean+Std","Mean-Std"};
    sea = new TimeSeries(ts.getDates(),data,fn,ts.getFieldUnits());
    String[] tsFieldNames = ts.getFieldNames();
    //Set short description.
    //This code is temporarily,
    sea.setShortDescription(dboName + " " + tsFieldNames[0]);
    //shall be replaced with this code.
//    sea.setShortDescription(ts.getShortDescription() + " " + tsFieldNames[0]);
  }

/*  private void getAllTimeSeries()
  {
    seaDD.setInfo(new String("Loading event 1 ..."));
    long eventTime = eventDates[0].getTime();
    Date start = new Date(eventTime + startWindow);
    Date end = new Date(eventTime + endWindow);
    TimeSeries epoch = getTimeSeries(start,end);
    Jama.Matrix epochData = epoch.getData();
    Jama.Matrix oneField = epochData.getMatrix(0,epochData.getRowDimension()-1,field,field);
    Jama.Matrix data = new Jama.Matrix(epoch.getLength(),eventDates.length,Double.NaN);
    data.setMatrix(0,epoch.getLength()-1,0,0,oneField);
    // Get the rest of the events.
    for (int i=1;i<eventDates.length;i++)
    {
      seaDD.setInfo(new String("Loading event " + (new Integer(i+1)).toString() + " ..."));
      eventTime = eventDates[i].getTime();
      start = new Date(eventTime + startWindow);
      end = new Date(eventTime + endWindow);
      epoch = getTimeSeries(start,end);
      epochData = epoch.getData();
      oneField = epochData.getMatrix(0,epochData.getRowDimension()-1,field,field);
      data.setMatrix(0,epoch.getLength()-1,i,i,oneField);
    }
    // Create the SEA time series.
    Date[] seaDates = new Date[epoch.getLength()];
    for (int i=0;i<seaDates.length;i++)
      seaDates[i] = new Date((endWindow-startWindow)/seaDates.length*i + startWindow);
    sea = new TimeSeries(seaDates,data);
  }*/

  private TimeSeries getTimeSeries(Date sd, Date ed)
  {
    TimeSeries res = null;
    if (!seaDD.isCancelled())
    {
      try
      {
        res = dbRMI.getTimeSeries(dboName,sd,ed);
      }
      catch(RemoteException re)
      {
        System.err.println(re);
        return null;
      }
      catch(IOException ioe)
      {
        System.err.println(ioe);
        return null;
      }
    }
 //   System.out.println("Read " + res.getLength() + " data points ...");
    return res;
  }

  public TimeSeries getTimeSeries()
  {
    return sea;
  }

/*  public void start()
  {
    thread = new Thread(this);
    thread.start();
  }

  public void run()
  {
    isRunning = true;
    getAllTimeSeries();
    isRunning = false;
  }

  public boolean isRunning()
  {
    return isRunning;
  } */

}
