package saaps.saam;

import java.io.IOException;
import java.util.Date;

import saaps.dbt.*;
import saaps.math.*;
import saaps.util.*;

/**
 *
 * @author
 * @version
 */
public class SEADataLocal extends Object
{
    //The uploaded time series.
    private TimeSeries sea;
    //The event dates.
    private Date[] eventDates;
    //Start and end windows.
    private long startWindow, endWindow;
    //Name of the database object.
    private String dboName;
    //Field to use.
    private int field;
    //The data dialog.
    SEADataDialog seaDD;
    //The thread to this object.
    //  Thread thread;
    //Flag if thread is running.
    //  boolean isRunning;
    
    public SEADataLocal(Date[] ed,long sw,long ew,String dbo,int f,SEADataDialog seadd) throws TimeSeriesIndexException, IllegalTimeSeriesException, IOException {
        eventDates = ed;
        startWindow = sw;
        endWindow = ew;
        dboName = dbo;
        field = f;
        seaDD = seadd;
        //If number of events is less than 4 then load each event individually,
        //else load only mean and standard deviation.
        if (eventDates.length<4)
            getAllTimeSeries();
        else
            getSEATimeSeries();
    }
    
    private void getAllTimeSeries() throws IOException {
        //The individual time series objects.
        TimeSeries[] ts = new TimeSeries[eventDates.length];
        //Load the time series.
//        Database db = Database.getInstance(dboName,true);
        ODIDatabase db = new ODIDatabase();
        for (int i=0;i<ts.length;i++) {
            long eventTime = eventDates[i].getTime();
            Date sd = new Date(eventTime + startWindow);
            Date ed = new Date(eventTime + endWindow);
            ts[i] = db.retrieveTS(dboName,sd,ed);
        }
//        db.close();
        //Create the SEA time series containing only the selected field.
        //Create the dates.
        Date[] dates = new Date[ts[0].getLength()];
        for (int row=0;row<dates.length;row++)
            dates[row] = new Date(ts[0].getDate(row).getTime()-eventDates[0].getTime());
        //Create the data.
        double[][] data = new double[ts[0].getLength()][eventDates.length];
        for (int row=0;row<data.length;row++)
            for(int col=0;col<data[0].length;col++)
                data[row][col] = ts[col].getData(row,field);
        //Create the field names and field units.
        String[] fn = new String[eventDates.length];
        String[] fu = new String[eventDates.length];
        for (int i=0;i<fn.length;i++) {
            int j=i+1;
            fn[i] = new String("Event " + j);
            String[] fus = ts[i].getFieldUnits();
            fu[i] = new String(fus[field]);
        }
        //Create the time series.
        sea = new TimeSeries(dates,data,fn,fu);
        String[] tsFieldNames = ts[0].getFieldNames();
        //Set short description.
        //This code is temporarily,
        sea.setShortDescription(dboName + " " + tsFieldNames[field]);
        //shall be replaced with this code.
        //    sea.setShortDescription(ts[0].getShortDescription() + " " + tsFieldNames[field]);
    }
    
    private void getSEATimeSeries() throws TimeSeriesIndexException,
    IllegalTimeSeriesException, IOException {
        //The loaded SEA time series.
        TimeSeries ts = null;
System.out.println(dboName);
        ODIDatabase db = new ODIDatabase();
//        Database db = Database.getInstance(dboName,true);
        ts = db.retrieveSEA(dboName,field,eventDates,startWindow,endWindow,true);
//        db.close();
        //Create time series for plotting.
        //The fields are: mean, mean+std, mean-std.
        double[][] data = new double[ts.getLength()][3];
        for (int row=0;row<data.length;row++) {
            data[row][0] = ts.getData(row,0);
            data[row][1] = data[row][0] + ts.getData(row,1);
            data[row][2] = data[row][0] - ts.getData(row,1);
        }
        //Set field names and field units.
        String[] fn = {"Mean","Mean+Std","Mean-Std"};
        sea = new TimeSeries(ts.getDates(),data,fn,ts.getFieldUnits());
        String[] tsFieldNames = ts.getFieldNames();
        //Set short description.
        //This code is temporarily,
        sea.setShortDescription(dboName + " " + tsFieldNames[0]);
        //shall be replaced with this code.
        //    sea.setShortDescription(ts.getShortDescription() + " " + tsFieldNames[0]);
    }
    
    private TimeSeries getTimeSeries(Date sd, Date ed) {
        TimeSeries res = null;
        if (!seaDD.isCancelled()) {
            try {
                Database db = Database.getInstance(dboName,true);
                res = db.retrieveTS(sd,ed,false);
                db.close();
            }
            catch(IOException ioe)
            {
                System.err.println(ioe);
                return null;
            }
            catch(Exception re)
            {
                System.err.println(re);
                return null;
            }
        }
        //   System.out.println("Read " + res.getLength() + " data points ...");
        return res;
    }
    
    public TimeSeries getTimeSeries() {
        return sea;
    }
}
