/*
  AnomPredApplet.java

  Title:      Anomaly Prediction
  Author:      Peter Wintoft
  Description:
*/

package saaps.sapm;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import java.util.TimeZone;
import saaps.dbt.DatabaseRMI;
import saaps.network.NetworkParam;
import saaps.network.NormParam;
import saaps.util.TimeSeries;
import saaps.util.YMDChoice;

public class AnomPredApplet extends Applet
{

  NetworkParam netPar = null;
  NormParam normPar = null;

// IMPORTANT: Source code between BEGIN/END comment pair will be regenerated
// every time the form is saved. All manual changes will be overwritten.
// BEGIN GENERATED CODE
  // member declarations
  java.awt.Label labelStart = new java.awt.Label();
  java.awt.Label labelSY = new java.awt.Label();
  java.awt.Label labelSM = new java.awt.Label();
  java.awt.Label labelSD = new java.awt.Label();
  java.awt.Choice choiceSY = new java.awt.Choice();
  java.awt.Choice choiceSM = new java.awt.Choice();
  java.awt.Choice choiceSD = new java.awt.Choice();
  java.awt.Label labelEnd = new java.awt.Label();
  java.awt.Label labelEY = new java.awt.Label();
  java.awt.Label labelEM = new java.awt.Label();
  java.awt.Label labelED = new java.awt.Label();
  java.awt.Choice choiceEY = new java.awt.Choice();
  java.awt.Choice choiceEM = new java.awt.Choice();
  java.awt.Choice choiceED = new java.awt.Choice();
  java.awt.Panel panelPlot = new java.awt.Panel();
  java.awt.Button buttonPlot = new java.awt.Button();
// END GENERATED CODE

//  boolean isStandalone = false;

  public AnomPredApplet()
  {
  }

  // Retrieve the value of an applet parameter
//  public String getParameter(String key, String def)
//  {
//    return isStandalone ? System.getProperty(key, def) :
//      (getParameter(key) != null ? getParameter(key) : def);
//  }

  // Get info on the applet parameters
  public String[][] getParameterInfo()
  {
    String[][] info = {
    {"Model", "String", "The name of the model"}};
    return info;
  }

  // Get applet information
  public String getAppletInfo()
  {
    return "Applet Information";
  }

  // Initialize the applet
  public void init()
  {
    showStatus("Initializing ...");
    super.init();
    // Set values in choices.
    // This should be initialized from the model definition.
    setChoiceValues();
    try {
      initComponents();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    System.out.println(getAppletInfo());
    String[][] info = getParameterInfo();
    for (int i=0;i<info.length;i++)
    {
      for (int j=0;j<3;j++)
        System.out.print(info[i][j]);
      System.out.println();
    }
    System.out.println(getParameter("Model"));

    //Create a default plot.

    //Get start date.
    YMDChoice ymd = new YMDChoice(choiceSY,choiceSM,choiceSD,0,0,0);
    Date sd = ymd.getDate();

    //Get end date.
    ymd.setChoice(choiceEY,choiceEM,choiceED,21,0,0);
    Date ed = ymd.getDate();

    //Get the data from the database.
    showStatus("Loading model and data ...");
    DatabaseRMI dbrmi = null;
    TimeSeries ts = null;
    System.out.println("//" + getCodeBase().getHost() +
        ":2001" + "/DatabaseRMIServer");
    try
    {
      dbrmi = (DatabaseRMI)
        Naming.lookup("//" + getCodeBase().getHost() +
        ":2001" + "/DatabaseRMIServer");
      ts = dbrmi.getTimeSeries("KpDB",sd,ed);
      netPar = dbrmi.getNetworkParam("SumKpNetwork.net");
      normPar = dbrmi.getNormParam("SumKpNetwork.norm",1);
      System.out.println("AnomPredApplet: netPar.getNumOfLayers()=" + netPar.getNumOfLayers());
    }
    catch(NotBoundException nbe)
    {
      System.err.println(nbe);
    }
    catch(MalformedURLException mfurle)
    {
      System.err.println(mfurle);
    }
    catch(RemoteException re)
    {
      System.err.println(re);
    }
    catch(IOException re)
    {
      System.err.println(re);
    }

    //Generate plot.
/*    showStatus("Running model and creating plot ...");
    AnomPredPlot app = new AnomPredPlot(netPar,normPar,ts);

    //Remove old plot.
    if (panelPlot.getComponentCount() > 0)
      panelPlot.remove(0);
    
    //Add plot to panel.
    panelPlot.add(app.getPlot());
//    setVisible(true);
//    panelPlot.setVisible(true);
//    panelPlot.repaint();
//    panelPlot.setEnabled(true);
//    this.repaint();
//    addNotify();
//    setEnabled(true);
*/    showStatus("Ready!");
  }

  public void initComponents() throws Exception
  {
// IMPORTANT: Source code between BEGIN/END comment pair will be regenerated
// every time the form is saved. All manual changes will be overwritten.
// BEGIN GENERATED CODE
    // the following code sets the frame's initial state
    labelStart.setText("Start");
    labelStart.setLocation(new java.awt.Point(21, 23));
    labelStart.setAlignment(Label.CENTER);
    labelStart.setVisible(true);
    labelStart.setSize(new java.awt.Dimension(104, 23));

    labelSY.setText("Year");
    labelSY.setLocation(new java.awt.Point(20, 54));
    labelSY.setVisible(true);
    labelSY.setSize(new java.awt.Dimension(40, 23));

    labelSM.setText("Month");
    labelSM.setLocation(new java.awt.Point(20, 80));
    labelSM.setVisible(true);
    labelSM.setSize(new java.awt.Dimension(40, 20));

    labelSD.setText("Day");
    labelSD.setLocation(new java.awt.Point(20, 105));
    labelSD.setVisible(true);
    labelSD.setSize(new java.awt.Dimension(40, 20));

    choiceSY.setLocation(new java.awt.Point(70, 50));
    choiceSY.setVisible(true);
    choiceSY.setSize(new java.awt.Dimension(60, 30));

    choiceSM.setLocation(new java.awt.Point(70, 75));
    choiceSM.setVisible(true);
    choiceSM.setSize(new java.awt.Dimension(60, 30));

    choiceSD.setLocation(new java.awt.Point(70, 100));
    choiceSD.setVisible(true);
    choiceSD.setSize(new java.awt.Dimension(60, 30));

    labelEnd.setText("End");
    labelEnd.setLocation(new java.awt.Point(21, 160));
    labelEnd.setAlignment(Label.CENTER);
    labelEnd.setVisible(true);
    labelEnd.setSize(new java.awt.Dimension(104, 23));

    labelEY.setText("Year");
    labelEY.setLocation(new java.awt.Point(20, 191));
    labelEY.setVisible(true);
    labelEY.setSize(new java.awt.Dimension(40, 23));

    labelEM.setText("Month");
    labelEM.setLocation(new java.awt.Point(20, 217));
    labelEM.setVisible(true);
    labelEM.setSize(new java.awt.Dimension(40, 20));

    labelED.setText("Day");
    labelED.setLocation(new java.awt.Point(20, 242));
    labelED.setVisible(true);
    labelED.setSize(new java.awt.Dimension(40, 20));

    choiceEY.setLocation(new java.awt.Point(70, 187));
    choiceEY.setVisible(true);
    choiceEY.setSize(new java.awt.Dimension(60, 30));

    choiceEM.setLocation(new java.awt.Point(70, 212));
    choiceEM.setVisible(true);
    choiceEM.setSize(new java.awt.Dimension(60, 30));

    choiceED.setLocation(new java.awt.Point(70, 237));
    choiceED.setVisible(true);
    choiceED.setSize(new java.awt.Dimension(60, 30));

    panelPlot.setLocation(new java.awt.Point(170, 40));
    panelPlot.setVisible(true);
    panelPlot.setSize(new java.awt.Dimension(450, 380));

    buttonPlot.setLocation(new java.awt.Point(52, 303));
    buttonPlot.setLabel("Plot");
    buttonPlot.setVisible(true);
    buttonPlot.setSize(new java.awt.Dimension(64, 23));

    setLocation(new java.awt.Point(0, 0));
    setLayout(null);
    setSize(new java.awt.Dimension(680, 453));
    add(labelStart);
    add(labelSY);
    add(labelSM);
    add(labelSD);
    add(choiceSY);
    add(choiceSM);
    add(choiceSD);
    add(labelEnd);
    add(labelEY);
    add(labelEM);
    add(labelED);
    add(choiceEY);
    add(choiceEM);
    add(choiceED);
    add(panelPlot);
    add(buttonPlot);


    buttonPlot.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent e) {
        buttonPlotActionPerformed(e);
      }
    });
// END GENERATED CODE
  }
  
  private void setChoiceValues()
  {
    int sy = 1970;
    for (int y=sy;y<2010;y++)
    {
      choiceSY.addItem(String.valueOf(y));
      choiceEY.addItem(String.valueOf(y));
    }
    for (int m=1;m<13;m++)
    {
      choiceSM.addItem(String.valueOf(m));
      choiceEM.addItem(String.valueOf(m));
    }
    for (int d=1;d<32;d++)
    {
      choiceSD.addItem(String.valueOf(d));
      choiceED.addItem(String.valueOf(d));
    }
    
    // Set default fields to today.
    Date d = new Date();
    TimeZone tz = TimeZone.getTimeZone("GMT");
    Calendar cs = Calendar.getInstance(tz);
    Calendar ce = Calendar.getInstance(tz);
    cs.setTime(d);
    cs.add(cs.DATE,-7);
    ce.setTime(d);
    choiceSY.select(cs.get(cs.YEAR)-sy);
    choiceSM.select(cs.get(cs.MONTH));
    choiceSD.select(cs.get(cs.DATE)-1);
    choiceEY.select(ce.get(ce.YEAR)-sy);
    choiceEM.select(ce.get(ce.MONTH));
    choiceED.select(ce.get(ce.DATE)-1);
  }
  
  public void buttonPlotActionPerformed(java.awt.event.ActionEvent e)
  {
    //Get start date.
    System.out.println("AnomPredApplet: " + choiceSY.getItem(choiceSY.getSelectedIndex()) );
    System.out.println("AnomPredApplet: " + choiceSM.getItem(choiceSM.getSelectedIndex()) );
    System.out.println("AnomPredApplet: " + choiceSD.getItem(choiceSD.getSelectedIndex()) );
    YMDChoice ymd = new YMDChoice(choiceSY,choiceSM,choiceSD,0,0,0);
    Date sd = ymd.getDate();
    
    //Move start date seven days back
    //to compensate for time delay line.
    Calendar calSD = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    calSD.setTime(sd);
    calSD.add(Calendar.DATE,-7);
    sd = calSD.getTime();
    
    //Get end date.
    ymd.setChoice(choiceEY,choiceEM,choiceED,21,0,0);
    Date ed = ymd.getDate();
    
    //Get the data from the database.
    showStatus("Reading the data ...");
    DatabaseRMI dbrmi = null;
    TimeSeries ts = null;
    try
    {
      dbrmi = (DatabaseRMI)
        Naming.lookup("//" + getCodeBase().getHost() + 
        ":2001" + "/DatabaseRMIServer");
      ts = dbrmi.getTimeSeries("KpDB",sd,ed);
    }
    catch(NotBoundException nbe)
    {
      System.err.println(nbe);
    }
    catch(MalformedURLException murle)
    {
      System.err.println(murle);
    }
    catch(RemoteException re)
    {
      System.err.println(re);
    }
    catch(IOException ioe)
    {
      System.err.println(ioe);
    }
    //Generate plot.
    showStatus("Running model ...");
    AnomPredPlot app = new AnomPredPlot(netPar,normPar,ts);
    
    //Remove old plot.
    if (panelPlot.getComponentCount() > 0)
      panelPlot.remove(0);
    
    //Add plot to panel.
    panelPlot.add(app.getPlot());
    
    //Update panel so that the new plot appears.
//    setVisible(true);
//    panelPlot.setVisible(true);
//    panelPlot.repaint();
//    panelPlot.setEnabled(true);
//    this.repaint();
//    addNotify();
//    setEnabled(true);
    panelPlot.validate();
    showStatus("Ready!");
  }
  
}
