/*
 * ElFluenceApplet.java
 *
 * Created on December 1, 2000, 2:56 PM
 */
 
package saaps.sapm;

import java.awt.Dimension;
import java.awt.Font;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import saaps.dbt.DatabaseRMI;
import saaps.util.HelpPage;
import saaps.util.TimeSeries;

/** 
 *
 * @author  saaps
 * @version 
 */
public class ElFluenceApplet extends java.applet.Applet {

  //The electron fluence.
  TimeSeries fluence;
  //The electron fluence plot.
  private ElFluencePlot3 elFluencePlot;
  //Calendar for UT time.
  Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
  //The minimum and maximum values of the y-axis.
  private double minY = 1e5;
  private double maxY = 1e9;
  //The data loading info.
  private java.awt.Label labelLoadingData = new java.awt.Label("Loading data ...", java.awt.Label.CENTER);
  //The object for creating the plot.
  private PlotCreator pc;

  /** Initializes the applet ElFluenceApplet */
  public void init () {
    initComponents ();
    labelLoadingData.setFont(new Font("Helvetica",Font.BOLD,24));
    textFieldYellow.setText(String.valueOf(5e7));
    textFieldRed.setText(String.valueOf(5e8));
    setDateTextFields(new Date());
    panelPlot.add(labelLoadingData);
    pc = new PlotCreator();
    (new Thread(pc)).start();
    setSize(610,430);
  }

  class PlotCreator implements Runnable
  {
    Date plotDate;
    double yl;
    double rl;
    
    PlotCreator()
    {
      plotDate = new Date();
      yl = 5e7;
      rl = 5e8;
      elFluencePlot = new ElFluencePlot3(400,300);
    }
    
    public void setDate(Date d)
    {
      plotDate = d;
    }
    
    public Date getDate()
    {
      return plotDate;
    }
    
    public void setYellowLevel(double y)
    {
      yl = y;
    }
    
    public void setRedLevel(double r)
    {
      rl = r;
    }
    
    public void run()
    {
      getFluence(plotDate);
      setDateTextFields();
      elFluencePlot.setData(fluence,yl,rl,minY,maxY);
      panelPlot.remove(labelLoadingData);
//      panelPlot.add(elFluencePlot,new com.netbeans.developer.awt.AbsoluteConstraints (0, 0, 400, 200));
      panelPlot.add(elFluencePlot);
      elFluencePlot.repaint();
//     panelPlot.repaint();
//      elFluencePlot.setVisible(true);
//      elFluencePlot.validate();
//      repaint();
//      validate();
//      panelPlot.remove(labelLoadingData);
//      panelPlot.add(elFluencePlot,new com.netbeans.developer.awt.AbsoluteConstraints (20, 20, -1, -1));
//      panelPlot.validate();
    }
  }
    
  private void getFluence(Date d)
  {
    DatabaseRMI dbrmi = null;
    try
    {
      dbrmi = (DatabaseRMI)
        Naming.lookup("//" + getCodeBase().getHost() +
        ":2001" + "/DatabaseRMIServer");
      fluence = dbrmi.retrieveElFluence(d);
    }
    catch(NotBoundException nbe)
    {
      System.err.println(nbe);
//      new ErrorDialog(getAppletFrame(),nbe.getMessage());
    }
    catch(MalformedURLException murle)
    {
      System.err.println(murle);
//      new ErrorDialog(getAppletFrame(),murle.getMessage());
    }
    catch(RemoteException re)
    {
      System.err.println(re);
//      new ErrorDialog(getAppletFrame(),re.getMessage());
    }
  }
  
  private void setDateTextFields(Date d)
  {
    cal.setTime(d);
    textFieldYear.setText(String.valueOf(cal.get(cal.YEAR)));
    textFieldMonth.setText(String.valueOf(cal.get(cal.MONTH)+1));
    textFieldDate.setText(String.valueOf(cal.get(cal.DATE)));
  }
  
  private void setDateTextFields()
  {
    Date d = fluence.getLastDate();
    setDateTextFields(d);
//    cal.setTime(d);
//    textFieldYear.setText(String.valueOf(cal.get(cal.YEAR)));
//    textFieldMonth.setText(String.valueOf(cal.get(cal.MONTH)+1));
//    textFieldDate.setText(String.valueOf(cal.get(cal.DATE)));
  }
  
  private double getTextFieldValue(java.awt.TextField tf)
  {
    double value = 0;
    try
    {
      value = Double.valueOf(tf.getText()).doubleValue();
    }
    catch(NumberFormatException nfe)
    {
      System.err.println(nfe);
    }
    return value;
  }
    
  public void setDefaultLevels()
  {
    textFieldYellow.setText(Double.toString(5e7));
    textFieldRed.setText(Double.toString(5e8));
  }
  
  public void drawLevels()
  {
    try
    {
      double levelRed = Double.valueOf(textFieldRed.getText()).doubleValue();
      double levelYellow = Double.valueOf(textFieldYellow.getText()).doubleValue();
      //Make sure that the red level is always higher than the yellow level.
      if(levelRed<levelYellow)
      {
        levelRed = levelYellow;
        textFieldRed.setText(Double.toString(levelRed));
      }
      elFluencePlot.setYellowLevel(levelYellow);
      elFluencePlot.setRedLevel(levelRed);
      elFluencePlot.repaint();
    }
    catch(NumberFormatException nfe)
    {
      System.err.println(nfe);
    }
  }
  
  /** This method is called from within the init() method to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    labelTitle = new java.awt.Label ();
    buttonHelp = new java.awt.Button ();
    labelLevels = new java.awt.Label ();
    labelRed = new java.awt.Label ();
    textFieldRed = new java.awt.TextField ();
    labelYellow = new java.awt.Label ();
    textFieldYellow = new java.awt.TextField ();
    buttonDefault = new java.awt.Button ();
    labelYear = new java.awt.Label ();
    textFieldYear = new java.awt.TextField ();
    labelMonth = new java.awt.Label ();
    textFieldMonth = new java.awt.TextField ();
    labelDate = new java.awt.Label ();
    textFieldDate = new java.awt.TextField ();
    buttonToday = new java.awt.Button ();
    buttonPlot = new java.awt.Button ();
    panelPlot = new java.awt.Panel ();
    setLayout (new com.netbeans.developer.awt.AbsoluteLayout ());
    setBackground (java.awt.Color.lightGray);

    labelTitle.setBackground (java.awt.Color.lightGray);
    labelTitle.setText ("24 hour fluence of GOES-8 >2 MeV electrons");
    labelTitle.setForeground (java.awt.Color.black);
    labelTitle.setName ("label6");
    labelTitle.setFont (new java.awt.Font ("Dialog", 1, 18));


    add (labelTitle, new com.netbeans.developer.awt.AbsoluteConstraints (20, 20, -1, -1));

    buttonHelp.setBackground (new java.awt.Color (204, 204, 204));
    buttonHelp.setForeground (java.awt.Color.black);
    buttonHelp.setLabel ("Help");
    buttonHelp.setName ("button5");
    buttonHelp.setFont (new java.awt.Font ("Dialog", 0, 11));
    buttonHelp.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        buttonHelpActionPerformed (evt);
      }
    }
    );


    add (buttonHelp, new com.netbeans.developer.awt.AbsoluteConstraints (500, 20, -1, -1));

    labelLevels.setBackground (java.awt.Color.lightGray);
    labelLevels.setText ("Threshold levels");
    labelLevels.setForeground (java.awt.Color.black);
    labelLevels.setName ("label7");
    labelLevels.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelLevels, new com.netbeans.developer.awt.AbsoluteConstraints (20, 80, -1, -1));

    labelRed.setBackground (java.awt.Color.lightGray);
    labelRed.setText ("Red");
    labelRed.setForeground (java.awt.Color.black);
    labelRed.setAlignment (java.awt.Label.RIGHT);
    labelRed.setName ("label8");
    labelRed.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelRed, new com.netbeans.developer.awt.AbsoluteConstraints (20, 130, -1, -1));

    textFieldRed.setBackground (new java.awt.Color (255, 247, 233));
    textFieldRed.setText ("5.0E8");
    textFieldRed.setForeground (java.awt.Color.black);
    textFieldRed.setName ("textfield3");
    textFieldRed.setFont (new java.awt.Font ("Dialog", 0, 11));
    textFieldRed.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        textFieldRedActionPerformed (evt);
      }
    }
    );


    add (textFieldRed, new com.netbeans.developer.awt.AbsoluteConstraints (70, 130, 80, -1));

    labelYellow.setBackground (java.awt.Color.lightGray);
    labelYellow.setText ("Yellow");
    labelYellow.setForeground (java.awt.Color.black);
    labelYellow.setAlignment (java.awt.Label.RIGHT);
    labelYellow.setName ("label9");
    labelYellow.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelYellow, new com.netbeans.developer.awt.AbsoluteConstraints (10, 160, -1, -1));

    textFieldYellow.setBackground (new java.awt.Color (255, 247, 233));
    textFieldYellow.setText ("5.0E7");
    textFieldYellow.setForeground (java.awt.Color.black);
    textFieldYellow.setName ("textfield4");
    textFieldYellow.setFont (new java.awt.Font ("Dialog", 0, 11));
    textFieldYellow.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        textFieldYellowActionPerformed (evt);
      }
    }
    );


    add (textFieldYellow, new com.netbeans.developer.awt.AbsoluteConstraints (70, 160, 80, -1));

    buttonDefault.setBackground (new java.awt.Color (204, 204, 204));
    buttonDefault.setForeground (java.awt.Color.black);
    buttonDefault.setLabel ("Default levels");
    buttonDefault.setName ("button6");
    buttonDefault.setFont (new java.awt.Font ("Dialog", 0, 11));
    buttonDefault.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        buttonDefaultActionPerformed (evt);
      }
    }
    );


    add (buttonDefault, new com.netbeans.developer.awt.AbsoluteConstraints (40, 200, -1, -1));

    labelYear.setBackground (java.awt.Color.lightGray);
    labelYear.setText ("Year");
    labelYear.setForeground (java.awt.Color.black);
    labelYear.setName ("label10");
    labelYear.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelYear, new com.netbeans.developer.awt.AbsoluteConstraints (20, 260, -1, -1));

    textFieldYear.setBackground (new java.awt.Color (255, 247, 233));
    textFieldYear.setText ("2000");
    textFieldYear.setForeground (java.awt.Color.black);
    textFieldYear.setName ("textfield5");
    textFieldYear.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (textFieldYear, new com.netbeans.developer.awt.AbsoluteConstraints (70, 260, 60, -1));

    labelMonth.setBackground (java.awt.Color.lightGray);
    labelMonth.setText ("Month");
    labelMonth.setForeground (java.awt.Color.black);
    labelMonth.setName ("label11");
    labelMonth.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelMonth, new com.netbeans.developer.awt.AbsoluteConstraints (20, 300, -1, -1));

    textFieldMonth.setBackground (new java.awt.Color (255, 247, 233));
    textFieldMonth.setText ("12");
    textFieldMonth.setForeground (java.awt.Color.black);
    textFieldMonth.setName ("textfield6");
    textFieldMonth.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (textFieldMonth, new com.netbeans.developer.awt.AbsoluteConstraints (70, 300, 60, -1));

    labelDate.setBackground (java.awt.Color.lightGray);
    labelDate.setText ("Date");
    labelDate.setForeground (java.awt.Color.black);
    labelDate.setName ("label12");
    labelDate.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (labelDate, new com.netbeans.developer.awt.AbsoluteConstraints (20, 340, -1, -1));

    textFieldDate.setBackground (new java.awt.Color (255, 247, 233));
    textFieldDate.setText ("01");
    textFieldDate.setForeground (java.awt.Color.black);
    textFieldDate.setName ("textfield7");
    textFieldDate.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (textFieldDate, new com.netbeans.developer.awt.AbsoluteConstraints (70, 340, 60, -1));

    buttonToday.setBackground (new java.awt.Color (204, 204, 204));
    buttonToday.setForeground (java.awt.Color.black);
    buttonToday.setLabel ("Today");
    buttonToday.setName ("button7");
    buttonToday.setFont (new java.awt.Font ("Dialog", 0, 11));
    buttonToday.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        buttonTodayActionPerformed (evt);
      }
    }
    );


    add (buttonToday, new com.netbeans.developer.awt.AbsoluteConstraints (20, 390, -1, -1));

    buttonPlot.setBackground (new java.awt.Color (204, 204, 204));
    buttonPlot.setForeground (java.awt.Color.black);
    buttonPlot.setLabel ("Plot");
    buttonPlot.setName ("button8");
    buttonPlot.setFont (new java.awt.Font ("Dialog", 0, 11));
    buttonPlot.addActionListener (new java.awt.event.ActionListener () {
      public void actionPerformed (java.awt.event.ActionEvent evt) {
        buttonPlotActionPerformed (evt);
      }
    }
    );


    add (buttonPlot, new com.netbeans.developer.awt.AbsoluteConstraints (100, 390, -1, -1));

    panelPlot.setBackground (new java.awt.Color (204, 204, 204));
    panelPlot.setForeground (java.awt.Color.black);
    panelPlot.setName ("panel27");
    panelPlot.setFont (new java.awt.Font ("Dialog", 0, 11));


    add (panelPlot, new com.netbeans.developer.awt.AbsoluteConstraints (180, 90, 410, 310));

  }//GEN-END:initComponents

  private void buttonHelpActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonHelpActionPerformed
// Add your handling code here:
    java.applet.AppletContext ac = getAppletContext();
    java.net.URL url = getDocumentBase();
    try
    {
      java.net.URL urlDoc = new java.net.URL(url,"ElFluenceAppletHelp.html");
      ac.showDocument(urlDoc,"Electron Fluence Help");
    }
    catch (java.net.MalformedURLException murle)
    {
      System.err.println(murle);
    }
//    new HelpPage(this,"ElFluenceAppletHelp.html");
  }//GEN-LAST:event_buttonHelpActionPerformed

  private void buttonPlotActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonPlotActionPerformed
// Add your handling code here:
    try
    {
      int year = Integer.valueOf(textFieldYear.getText()).intValue();
      int month = Integer.valueOf(textFieldMonth.getText()).intValue()-1;
      int date = Integer.valueOf(textFieldDate.getText()).intValue();
      cal.setTime(pc.getDate());
      //If dates have not changed just redraw the levels.
      if(cal.get(cal.YEAR)==year & cal.get(cal.MONTH)==month & cal.get(cal.DATE)==date)
      {
        drawLevels();
      }
      //If new dates load new data.
      else
      {
        panelPlot.remove(elFluencePlot);
        panelPlot.add(labelLoadingData);
        cal.set(cal.YEAR,year);
        cal.set(cal.MONTH,month);
        cal.set(cal.DATE,date);
        cal.set(cal.HOUR_OF_DAY,23);
        cal.set(cal.MINUTE,59);
        double levelRed = Double.valueOf(textFieldRed.getText()).doubleValue();
        double levelYellow = Double.valueOf(textFieldYellow.getText()).doubleValue();
        pc.setDate(cal.getTime());
        pc.setYellowLevel(levelYellow);
        pc.setRedLevel(levelRed);
        (new Thread(pc)).start();
      }
    }
    catch(NumberFormatException nfe)
    {
      System.err.println(nfe);
    }
  }//GEN-LAST:event_buttonPlotActionPerformed

  private void buttonTodayActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonTodayActionPerformed
// Add your handling code here:
    setDateTextFields(new Date());
    repaint();
  }//GEN-LAST:event_buttonTodayActionPerformed

  private void buttonDefaultActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonDefaultActionPerformed
// Add your handling code here:
    setDefaultLevels();
    drawLevels();
  }//GEN-LAST:event_buttonDefaultActionPerformed

  private void textFieldYellowActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textFieldYellowActionPerformed
// Add your handling code here:
    drawLevels();
  }//GEN-LAST:event_textFieldYellowActionPerformed

  private void textFieldRedActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textFieldRedActionPerformed
// Add your handling code here:
    drawLevels();
  }//GEN-LAST:event_textFieldRedActionPerformed


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private java.awt.Label labelTitle;
  private java.awt.Button buttonHelp;
  private java.awt.Label labelLevels;
  private java.awt.Label labelRed;
  private java.awt.TextField textFieldRed;
  private java.awt.Label labelYellow;
  private java.awt.TextField textFieldYellow;
  private java.awt.Button buttonDefault;
  private java.awt.Label labelYear;
  private java.awt.TextField textFieldYear;
  private java.awt.Label labelMonth;
  private java.awt.TextField textFieldMonth;
  private java.awt.Label labelDate;
  private java.awt.TextField textFieldDate;
  private java.awt.Button buttonToday;
  private java.awt.Button buttonPlot;
  private java.awt.Panel panelPlot;
  // End of variables declaration//GEN-END:variables

}