/*
 * Model.java
 *
 * Created on November 7, 2000, 3:35 PM
 */
 
package saaps.sapm;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import saaps.ann.*;
import saaps.ann.mlff.*;
import saaps.util.PlotFrame;
import saaps.util.TimeSeries;
import saaps.util.TimeSeriesPlot;

/** 
 *
 * @author  saaps
 * @version 
 */
public abstract class Model extends Object {
  
  //The input time series.
  protected TimeSeries inputTS;
  //The input data.
  protected Data inputData;
  //The output data.
  protected Data outputData;
  //The output time series.
  protected TimeSeries outputTS;
  //The neural network.
  protected Network ann;
  //The start date of the prediction.
  protected Date d1pred;
  //The end date of the prediction.
  protected Date d2pred;
  //The start date of data to be loaded.
  protected Date d1load;
  //The end date of the data to be loaded.
  protected Date d2load;
  //A calendar object to be used for time computations.
  private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

  /** Creates new Model */
  public Model() {
  }
  
  /**
  * This method creates the network.
  */
  public abstract void createNetwork();
  
  /**
  * Load the data that are needed for the input.
  */
  public abstract void loadData();
  
  /**
  * Pre-process the data.
  */
  public abstract void preProcess();
  
  /**
  * Run the network.
  */
  public void run()
  {
    ann.setInput(inputData);
    outputData = ann.getOutput();
  }
  
  /**
  * Post-process the data.
  */
  public abstract void postProcess();
  
  /**
  * Set the start and end dates of the prediction.
  *
  * @param start  The start date
  * @param end    The end date
  */
  public void setPredictionDates(Date start, Date end)
  {
    d1pred = new Date(start.getTime());
    d2pred = new Date(end.getTime());
  }
  
  /**
  * Add time to a date object.
  *
  * @param  d       The date
  * @param  field   Time field (@see Calendar)
  * @param  amount  The amount to add
  */
  protected Date addDate(Date d, int field, int amount)
  {
    cal.setTime(d);
    cal.add(field,amount);
    return cal.getTime();
  }
  
  /**
  * Plot the output.
  */
  public void plot()
  {
    PlotFrame plot1 = new PlotFrame(inputTS);
    TimeSeriesPlot tsp = new TimeSeriesPlot(outputTS);
    tsp.setZeroTime(outputTS.getLastDate());
    PlotFrame plot2 = new PlotFrame(tsp);
    plot1.show();
    plot2.show();
  }

}