package saaps.util;



import java.awt.*;              //for layout managers
import java.awt.event.*;        //for action and window events

import javax.swing.* ;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.* ;
import javax.swing.text.html.* ;

import java.io.* ;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Vector;
import java.util.Iterator;

/**
 * AppHelp displays a help page from an URL.
 * Normally a local html file.
 */
public class AppHelp extends JFrame implements HyperlinkListener {

    //EditorKit editorKit;
    //JTextPane textPane;
    JEditorPane editorPane;
    JButton back;
    URL helpURL;
    
    Vector urlList = new Vector(10);
    
    public AppHelp(String helpFile) {
        super("Help") ;
        getContentPane().setLayout(new java.awt.BorderLayout());
        
        /* We need a back button */
        back = new JButton();
        
// Not in use
/*        String imagefile = 
                System.getProperty("saaps.home")
                + "/lib/saaps/back.gif";

        back.setIcon(new javax.swing.ImageIcon(imagefile));
*/
        back.setRequestFocusEnabled(false);
        back.setMargin(new Insets(1,1,1,1));

        back.setFont(new java.awt.Font("Dialog", 0, 11));
        back.setLabel("Back");
        back.setForeground(java.awt.Color.black);
        
        back.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backActionPerformed(evt);
            }
        });
        
        JPanel toolBarPanel = new JPanel();
        toolBarPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        toolBarPanel.add(back);
        getContentPane().add(toolBarPanel,java.awt.BorderLayout.NORTH);
        
        //Create an editor pane.
        editorPane = createEditorPane(helpFile);
        urlList.add(helpURL); // The createEditorPane will setup the helpURL

        JScrollPane editorScrollPane = new JScrollPane(editorPane);
        editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        editorScrollPane.setPreferredSize(new Dimension(250, 145));
        editorScrollPane.setMinimumSize(new Dimension(10, 10));
        editorPane.addHyperlinkListener(this);
        getContentPane().add(editorScrollPane,java.awt.BorderLayout.CENTER);
    }

    public static AppHelp getAppHelp(String helpFile) {
        final AppHelp he = new AppHelp(AppHelp.formatClassName(helpFile)) ;
        
        he.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                //System.exit(0);
                he.dispose();
            }
        });
        
        he.setSize(900,600) ;
        he.setVisible(true) ;
        return he;
    }

    /**
     * Replaces the dots in the classname with file.separator
     * @param cn the classname
     * @return the complete "filename"
     */
     public static String formatClassName(String cn) {
        return cn.replace('.',(System.getProperty("file.separator")).charAt(0));
    }
    
    public static void main(String[] args) {
        try {
            AppHelp he = new AppHelp(AppHelp.formatClassName(args[0])) ;
            
            he.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
            
            he.setSize(400,400) ;
            he.setVisible(true) ;
        }
        catch (Exception e) {
            System.err.println(e);
        }
    }

    private JEditorPane createEditorPane(String helpFile) {
        editorPane = new JEditorPane();
        editorPane.setEditable(false);
        String s = null;
        try {
            s = "file:"
                + System.getenv("SAAPS_HOME")
                + System.getProperty("file.separator")
                + "lib"
                + System.getProperty("file.separator")
                + helpFile
                + "Help.html";
            helpURL = new URL(s);
            displayURL(helpURL, editorPane);
        } catch (Exception e) {
            System.err.println("Couldn't create help URL: " + s);
        }

        return editorPane;
    }

    private void displayURL(URL url, JEditorPane editorPane) {
        try {
            editorPane.setPage(url);
        } catch (IOException e) {
            System.err.println("Attempted to read a bad URL: " + url);
        }
    }

    public void hyperlinkUpdate(HyperlinkEvent hyperlinkEvent) {
        HyperlinkEvent.EventType type = hyperlinkEvent.getEventType();
        if(type == HyperlinkEvent.EventType.ACTIVATED) {
            urlList.add(hyperlinkEvent.getURL());
            displayURL(((URL) urlList.lastElement()),editorPane);
            Iterator it = urlList.iterator();
            while ( it.hasNext())
                System.out.println(((URL) it.next()));
        }
    }
    
    private void backActionPerformed(java.awt.event.ActionEvent evt) {
        if(urlList.size()>1) {
            urlList.remove(urlList.size()-1);
            urlList.trimToSize();
            displayURL(((URL) urlList.elementAt(urlList.size()-1)), editorPane);
        }
        else
            displayURL(((URL) urlList.lastElement()),editorPane);
    }
}
