FUNCTION cdf_date2cdf_epoch, date

  epoch = -1.0D0

  months = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']

  ddate = STRSPLIT(date, /EXTRACT)
  cdate = STRSPLIT(ddate[0], '-', /EXTRACT)
  day = FIX(cdate[0])
  month = WHERE(months EQ cdate[1])
  month = month[0] + 1
  year = FIX(cdate[2])
  ctime = STRSPLIT(ddate[1], ':', /EXTRACT)
  hour = FIX(ctime[0])
  minute = FIX(ctime[1])
  secs = STRSPLIT(ctime[2], '.', /EXTRACT)
  second = FIX(secs[0])
  IF (N_ELEMENTS(secs) GT 1) THEN msecs = FIX(secs[1]) ELSE msecs = 0

  cdf_epoch, epoch, year, month, day, hour, minute, second, msecs, /COMPUTE_EPOCH

  return, epoch

END


FUNCTION sql_date2cdf_epoch, date

  epoch = -1.0D0

  cdf_epoch, epoch, FIX(STRMID(date, 0, 4)), FIX(STRMID(date, 5, 2)), $
             FIX(STRMID(date, 8, 2)), FIX(STRMID(date, 11, 2)), $
             FIX(STRMID(date, 14, 2)), FIX(STRMID(date, 17, 2)), $
             FIX(STRMID(date, 20, 3)), /COMPUTE_EPOCH

  return, epoch

END


PRO error_exit, message

  PRINT, message
  EXIT, STATUS=1

END


FUNCTION create_var, cdf_id, var

  stmt = "id = CDF_VARCREATE(cdf_id, '" + var.name  + "'"

  IF (var.dims EQ 0) THEN BEGIN
    stmt += ", ['VARY']"
  ENDIF ELSE BEGIN
    stmt += ", ["
    rvars = STRSPLIT(var.dvar, ",", /EXTRACT)
    FOR i=0,var.dims-1 DO BEGIN
      IF (i GT 0) THEN stmt += ","
      IF (rvars[i] EQ 'T') THEN $
        stmt += "'VARY'" $
      ELSE $
        stmt += "'NOVARY'"
    ENDFOR
    stmt += "], DIMENSIONS=[" + $
            STRJOIN(REVERSE(STRSPLIT(var.sizes, ',', /EXTRACT)), ',') + "]"
  ENDELSE

  stmt += ", /" + var.cdf_type + $
          ", NUMELEM=" + STRING(var.number_elements) + $
          ", REC_NOVARY=" + STRING(FIX(var.rvar EQ 'F'), FORMAT='(I1)') + $
          ", REC_VARY=" + STRING(FIX(var.rvar EQ 'T'), FORMAT='(I1)') + $
          ", /ZVARIABLE)"

  res = EXECUTE(stmt)

  IF (res EQ 0) THEN error_exit, ' Error creating cdf variable ' + var.name

  return, id

END


FUNCTION CAST, invar, type

  IF (type EQ 'BYTE') THEN BEGIN
    outvar = BYTE(invar)
  ENDIF ELSE IF (type EQ 'INT') THEN BEGIN
    outvar = FIX(invar)
  ENDIF ELSE IF (type EQ 'UINT') THEN BEGIN
    outvar = UINT(invar)
  ENDIF ELSE IF (type EQ 'LONG') THEN BEGIN
    outvar = LONG(invar)
  ENDIF ELSE IF (type EQ 'ULONG') THEN BEGIN
    outvar = ULONG(invar)
  ENDIF ELSE IF (type EQ 'FLOAT') THEN BEGIN
    outvar = FLOAT(invar)
  ENDIF ELSE IF (type EQ 'DOUBLE') THEN BEGIN
    outvar = DOUBLE(invar)
  ENDIF ELSE IF (type EQ 'STRING') THEN BEGIN
    outvar = STRING(invar)
  ENDIF ELSE BEGIN
    error_exit, ' Variable type ' + type + ' is not a valid type!'
  ENDELSE

  return, outvar

END


PRO DBConnect, oDB

  status = DB_EXISTS()

  IF (~status) THEN error_exit, " Error starting IDL DataMiner"

  oDB = OBJ_NEW('IDLDBDatabase')

  ds = "ODI_SEDAT"
  sources = oDB->GetDataSources()
  dbindex = WHERE(sources.datasource EQ ds, n)
  IF (n LT 1) THEN error_exit, " Error connecting to " + ds

  oDB->Connect, DataSource=ds

END


PRO odicdf, cdf_name, dataset_name, varnames=varnames, $
            epochrange=epochrange, recordrange=recordrange

  dataset_name = STRSPLIT(dataset_name, 'SYSTEM!', /EXTRACT, /REGEX)

  IF (STRPOS(cdf_name, '.cdf') LT 0) THEN cdf_name += '.cdf'
  FILE_DELETE, cdf_name, /ALLOW_NONEXISTENT

  IF ((N_ELEMENTS(epochrange) EQ 0) AND (N_ELEMENTS(recordrange) EQ 0)) THEN BEGIN
    error_exit, ' Either an epoch range or a record range should be specified!'
  ENDIF ELSE IF ((N_ELEMENTS(epochrange) GT 0) AND (N_ELEMENTS(recordrange) GT 0)) THEN BEGIN
    error_exit, ' Epoch range and record range cannot both be specified!'
  ENDIF ELSE IF (N_ELEMENTS(epochrange) GT 0) THEN BEGIN
    e1 = sql_date2cdf_epoch(epochrange[0])
    IF (e1 LT 0.0D0) THEN $
      error_exit, ' Invalid epoch: ' + epochrange[0]
    e2 = sql_date2cdf_epoch(epochrange[1])
    IF (e2 LT 0.0D0) THEN $
      error_exit, ' Invalid epoch: ' + epochrange[1]
    IF (e1 GT e2) THEN $
      error_exit, ' Last epoch should be greater than first!'
    sqlrange = ' WHERE cdf_epoch>=' + STRING(e1) + ' AND cdf_epoch <=' + STRING(e2)
  ENDIF ELSE BEGIN
    IF (recordrange[0] GT recordrange[1]) THEN $
      error_exit, ' Last record number should be greater than first!'
    sqlrange = ' LIMIT ' + STRING(recordrange[0]-1) + "," + $
               STRING(recordrange[1]-recordrange[0]+1)
  ENDELSE

; Connect to the database
  DBConnect, oDB

; Find the dataset id
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, $
                SQL="SELECT id, name, LEFT(majority, 20) AS majority " + $
                    "FROM dataset WHERE " + $
                    "sedat_name='" + STRUPCASE(dataset_name) + "'")
  IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
    ds = oRS->GetRecord()
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Dataset ' + dataset_name + ' not found!'

  cdf_id = CDF_CREATE(cdf_name, /CLOBBER, /SINGLE_FILE, /NETWORK_ENCODING, $
                      ROW_MAJOR=FIX(ds.majority EQ 'ROW'), $
                      COL_MAJOR=FIX(ds.majority NE 'ROW'))
;
; Retrieve and store global attributes from table dataset_metadata
;
  sql = "SELECT LEFT(dm.attribute, 100) AS name, " + $
        "LEFT(dt.idl_type, 20) AS type, dm.entry_number, " + $
        "LEFT(dm.value, 1000) AS value FROM dataset_metadata AS dm " + $
        "LEFT JOIN data_type AS dt ON (dm.data_type_id=dt.id) " + $
        "WHERE dm.dataset_id=" + STRING(ds.id) + " AND " + $
        "dm.attribute!='platform' AND dm.attribute!='platform_type' " + $
        "AND dm.attribute!='instrument'"
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
    att = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      att = [att, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading global attributes!'

  FOR i=0,N_ELEMENTS(att)-1 DO BEGIN
    IF (att[i].entry_number EQ 1) THEN $
      att_id = CDF_ATTCREATE(cdf_id, att[i].name, /GLOBAL_SCOPE)
    IF (att[i].name EQ 'Logical_file_id') THEN BEGIN
      aval = CAST(cdf_name, att[i].type)
    ENDIF ELSE IF (att[i].name EQ 'Generated_by') THEN BEGIN
      IF (!VERSION.OS_FAMILY EQ 'Windows') THEN $
        hostname = GETENV('Computername') $
      ELSE $
        hostname = GETENV('HOSTNAME')
      aval = CAST('ODI V1.0 on ' + hostname, att[i].type)
    ENDIF ELSE IF (att[i].name EQ 'Generation_date') THEN BEGIN
      aval = CAST(SYSTIME(/UTC), att[i].type)
    ENDIF ELSE BEGIN
      aval = CAST(att[i].value, att[i].type)
    ENDELSE
    CDF_ATTPUT, cdf_id, att_id, att[i].entry_number-1, aval
  ENDFOR

;
; Retrieve and store list of variable attributes from table variable_attribute
;
  sql = "SELECT LEFT(va.name, 100) AS name FROM " + $
        "variable_attribute AS va LEFT JOIN variable_metadata AS vm " + $
        "ON (va.variable_metadata_id=vm.id) " + $
        "WHERE vm.dataset_id=" + STRING(ds.id) + " GROUP BY va.name " + $
        "ORDER BY va.name"
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
    att = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      att = [att, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading variable attributes!'

  FOR i=0,N_ELEMENTS(att)-1 DO $
    att_id = CDF_ATTCREATE(cdf_id, att[i].name, /VARIABLE_SCOPE)


;
; Retrieve and store variable definitions from table variable_metadata
;
  sql = "SELECT vm.id, LEFT(vm.name, 100) AS name, vm.number_elements, " + $
        "vm.dims, LEFT(vm.sizes, 500) AS sizes, " + $
        "LEFT(vm.record_variance, 10) AS rvar, " + $
        "LEFT(vm.dimension_variances, 100) AS dvar, " + $
        "LEFT(dt.cdf_type, 20) AS cdf_type, " + $
        "LEFT(dt.idl_type, 20) AS idl_type FROM " + $
        "variable_metadata AS vm LEFT JOIN data_type AS dt " + $
        "ON (vm.data_type_id=dt.id) " + $
        "WHERE vm.dataset_id=" + STRING(ds.id)
  IF (N_ELEMENTS(varnames) GT 0) THEN BEGIN
    sql += " AND (vm.name='" + varnames[0] + "'"
    FOR i=1,N_ELEMENTS(varnames)-1 DO $
      sql += " OR vm.name='" + varnames[i] + "'"
    sql += ")"
  ENDIF
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
    var = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      var = [var, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading variable definitions!'
  IF ((N_ELEMENTS(varnames) GT 0) AND $
      (N_ELEMENTS(varnames) NE N_ELEMENTS(var))) THEN $
    error_exit, ' Not all variables were found!'

;
; Store variables
; Retrieve and store variable attributes from table variable_attribute
;
  FOR i=0,N_ELEMENTS(var)-1 DO BEGIN
    id = create_var(cdf_id, var[i])
    sql = "SELECT va.id, LEFT(va.name, 50) AS name, " + $
          "LEFT(va.value, 500) AS value, " + $
          "LEFT(dt.cdf_type, 20) AS cdf_type, " + $
          "LEFT(dt.idl_type, 20) AS idl_type FROM " + $
          "variable_attribute AS va LEFT JOIN data_type AS dt " + $
          "ON (va.data_type_id=dt.id) " + $
          "WHERE va.variable_metadata_id=" + STRING(var[i].id)
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
    IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
      att = oRS->GetRecord()
      WHILE (oRS->MoveCursor(/NEXT)) DO $
        att = [att, oRS->GetRecord()]
      OBJ_DESTROY, oRS
    ENDIF ELSE $
      error_exit, ' Error reading variable attributes for ' + var[i].name + '!'
    FOR j=0,N_ELEMENTS(att)-1 DO BEGIN
      IF (att[j].cdf_type EQ 'CDF_EPOCH') THEN BEGIN
        CDF_ATTPUT, cdf_id, att[j].name, var[i].name, $
                    cdf_date2cdf_epoch(att[j].value)
      ENDIF ELSE BEGIN
        CDF_ATTPUT, cdf_id, att[j].name, var[i].name, $
                    CAST(att[j].value, att[j].idl_type)
      ENDELSE
    ENDFOR
;
; For variables with record variance FALSE, write RV values
;
    IF (var[i].rvar EQ 'F') THEN BEGIN
      sql = "SELECT LEFT(vn.nrv_pos, 100) AS pos, " + $
            "LEFT(vn.nrv_value, 1000) AS value FROM variable_nrv AS vn " + $
            "WHERE vn.variable_metadata_id=" + STRING(var[i].id) + $
            " ORDER BY nrv_index"
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
      IF (oRS->MoveCursor(/FIRST) EQ 1) THEN BEGIN
        nrv = oRS->GetRecord()
        WHILE (oRS->MoveCursor(/NEXT)) DO $
          nrv = [nrv, oRS->GetRecord()]
        OBJ_DESTROY, oRS
      ENDIF ELSE $
        error_exit, ' Error reading RV values for ' + var[i].name + '!'
      IF (var[i].dims EQ 0) THEN BEGIN
        dnrv = STRARR(1)
      ENDIF ELSE BEGIN
        res = EXECUTE("dnrv = STRARR(" + $
                      STRJOIN(REVERSE(STRSPLIT(var[i].sizes, ',', /EXTRACT)), ',') + $
                      ")")
      ENDELSE
      nr = SIZE(dnrv)
      nr = nr[N_ELEMENTS(nr)-1]
      IF (nr NE N_ELEMENTS(nrv)) THEN $
        error_exit, ' Size of variable ' + var[i].name + $
                    ' does not match number of RV values!'
      FOR k=0,nr-1 DO dnrv[*] = nrv.value
;      IF (ds.majority EQ 'ROW') THEN dnrv = TRANSPOSE(dnrv)
      CDF_VARPUT, cdf_id, var[i].name, CAST(dnrv, var[i].idl_type)
    ENDIF
  ENDFOR


;
; Read the record varying variables from the database and
; write them to the cdf file
;

; Filter out the record varying variables
  ind = WHERE(var.rvar EQ 'T')
  IF (ind[0] ge 0) THEN BEGIN
    var = var[ind]
    nvar = N_ELEMENTS(var)
    vardesc = {vard, dtype: '', vtype: '', sql: '', ndims: 0L, dimensions: LONARR(10)}
    vardesc = REPLICATE(vardesc, nvar)
    FOR i=0,nvar-1 DO BEGIN
      IF (var[i].dims EQ 0) THEN BEGIN
        vardesc[i].vtype = 'STRARR(1)'
        IF (var[i].name EQ 'Epoch') THEN BEGIN
          vardesc[i].sql = 'cdf_epoch AS Epoch'
        ENDIF ELSE BEGIN
          IF (var[i].idl_type EQ 'STRING') THEN BEGIN
            vardesc[i].sql = 'LEFT(' + STRLOWCASE(var[i].name) + $
                             ', 1000) AS ' + var[i].name
          ENDIF ELSE BEGIN
            vardesc[i].sql = STRLOWCASE(var[i].name) + ' AS ' + var[i].name
          ENDELSE
        ENDELSE
      ENDIF ELSE IF (var[i].dims EQ 1) THEN BEGIN
        vardesc[i].vtype = 'STRARR(' + var[i].sizes + ')'
        vardesc[i].sql = ''
        FOR j=0,var[i].sizes-1 DO BEGIN
          IF (j GT 0) THEN vardesc[i].sql += ','
          IF (var[i].idl_type EQ 'STRING') THEN BEGIN
            vardesc[i].sql += 'LEFT(' + STRLOWCASE(var[i].name) + '_' + $
                              STRTRIM(STRING(j+1), 2) + ', 1000) AS ' + $
                              var[i].name+ '_' + STRTRIM(STRING(j+1), 2)
          ENDIF ELSE BEGIN
            vardesc[i].sql += STRLOWCASE(var[i].name) + '_' + $
                              STRTRIM(STRING(j+1), 2) + ' AS ' + $
                              var[i].name+ '_' + STRTRIM(STRING(j+1), 2)
          ENDELSE
        ENDFOR
      ENDIF ELSE BEGIN
        vardesc[i].vtype = 'STRARR(' + $
                           STRJOIN(REVERSE(STRSPLIT(var[i].sizes, ',', /EXTRACT)), ',') + $
                           ')'
      ENDELSE
    ENDFOR
    sql = "SELECT "
    FOR i=0,nvar-1 DO BEGIN
      IF (i GT 0) THEN sql += ","
      sql += vardesc[i].sql
    ENDFOR
    sql += " FROM " + ds.name + sqlrange

    nrec = 0L
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, N_BUFFERS=1000, SQL=sql)
    status = oRS->MoveCursor(/FIRST)
    WHILE (status GT 0) DO BEGIN
      data = oRS->GetRecord()
      nr = 0
      FOR i=0,nvar-1 DO BEGIN
        res = EXECUTE("dvar = " + vardesc[i].vtype)
        dvar = CAST(dvar, var[i].idl_type)
        nd = SIZE(dvar)
        nd = nd[N_ELEMENTS(nd)-1]
        FOR j=0,nd-1 DO dvar[j] = data.(nr+j)
        CDF_VARPUT, cdf_id, var[i].name, dvar, REC_START=nrec
        nr += nd
      ENDFOR
      nrec += 1
      status = oRS->MoveCursor(/NEXT)
    ENDWHILE
    OBJ_DESTROY, oRS
  ENDIF

  CDF_CLOSE, cdf_id

  OBJ_DESTROY, oDB

END