FUNCTION cast_odi, var, type

  IF (type EQ 'C') THEN cvar = STRING(var) $
  ELSE IF (type EQ 'I') THEN cvar = LONG(var) $
  ELSE IF (type EQ 'R') THEN cvar = FLOAT(var) $
  ELSE IF (type EQ 'D') THEN cvar = DOUBLE(var) $
  ELSE cvar = 'error'

  RETURN, cvar

END


PRO DBConnect, oDB

  status = DB_EXISTS()

  IF (~status) THEN error_message, "Error starting IDL DataMiner"

  oDB = OBJ_NEW('IDLDBDatabase')

  ds = "ODI_SEDAT"
  sources = oDB->GetDataSources()
  dbindex = WHERE(sources.datasource EQ ds, n)
;  IF (n LT 1) THEN error_message, "Error connecting to " + ds

  oDB->Connect, DataSource=ds

END



PRO sedat_open_read_odi, datasetname, numents, varmeta, status, oDB, cmn=cmn

  status = 0

  DBConnect, oDB

  query = "SELECT id, name, epoch_start AS tmin, " + $
          "epoch_end AS tmax " + $
          "FROM dataset WHERE " + $
          "sedat_name='" + datasetname + "'"
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
  dataset = oRS->GetRecord()
  OBJ_DESTROY, oRS

  query = "SELECT CAST(COUNT(cdf_epoch) AS BINARY) AS np FROM " + dataset.name
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
  rec = oRS->GetRecord()
  numents = LONG(rec.np)
  OBJ_DESTROY, oRS

  query = "SELECT vm.id, CAST(vm.name AS CHAR) AS name, vm.dims, " + $
          "CAST(vm.sizes AS CHAR) AS sizes, " + $
          "LEFT(dt.sedat_type, 500) AS type FROM " + $
          "variable_metadata AS vm LEFT JOIN data_type AS dt ON " + $
          "(vm.data_type_id=dt.id) WHERE dataset_id=" + $
          STRING(dataset.id)
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
  vars = oRS->GetRecord()
  WHILE (oRS->MoveCursor(/NEXT)) DO $
    vars = [vars, oRS->GetRecord()]
  OBJ_DESTROY, oRS
  numcols = N_ELEMENTS(vars)

  varmeta = 0

  FOR i=0,numcols-1 DO BEGIN
    colname = vars[i].name
    datatype = vars[i].type

    ndims = LONG(vars[i].dims)
    IF (ndims EQ 0) THEN BEGIN
      ndims = 1L
      dimensions = [1L]
    ENDIF ELSE IF (ndims EQ 1) THEN BEGIN
      dimensions = [LONG(vars[i].sizes)]
    ENDIF ELSE BEGIN
      dimensions = LONG(STR_SEP(vars[i].sizes, ","))
    ENDELSE
    num_elements = 1
    FOR j=0,ndims-1 DO num_elements = num_elements * dimensions[j]

    units = ""
    query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND UPPER(name)='UNITS'"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    status = oRS->MoveCursor(/FIRST)
    IF (status EQ 1) THEN BEGIN
      rec = oRS->GetRecord()
      units = rec.value
    ENDIF
    OBJ_DESTROY, oRS

    null = "*"
    query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND UPPER(name)='FILLVAL'"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    status = oRS->MoveCursor(/FIRST)
    IF (status EQ 1) THEN BEGIN
      rec = oRS->GetRecord()
      null = cast_odi(rec.value, datatype)
    ENDIF
    OBJ_DESTROY, oRS

    si_rel = ""
    query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND UPPER(name)='SI_CONVERSION'"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    status = oRS->MoveCursor(/FIRST)
    IF (status EQ 1) THEN BEGIN
      rec = oRS->GetRecord()
      si_rel = rec.value
    ENDIF
    OBJ_DESTROY, oRS

; If a colname!LABLAXIS parameter exists then the value of this is retrieved
; to provide the axes_desc (axes description) details

    axis_label = ""
    query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND UPPER(name)='LABLAXIS'"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    status = oRS->MoveCursor(/FIRST)
    IF (status EQ 1) THEN BEGIN
      single_label = 1
      rec = oRS->GetRecord()
      axis_label = rec.value
    ENDIF ELSE BEGIN
      single_label = 0
      axis_label = colname
    ENDELSE
    OBJ_DESTROY, oRS
    axes_desc = axis_label

; The colname!VAR_TYPE parameter defines the type of the variable/column,
; whether data, support data or metadata

    var_type = ""
    query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND UPPER(name)='VAR_TYPE'"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    status = oRS->MoveCursor(/FIRST)
    IF (status EQ 1) THEN BEGIN
      rec = oRS->GetRecord()
      var_type = rec.value
    ENDIF
    OBJ_DESTROY, oRS

; For array-valued 'data' columns
    IF ((var_type EQ 'data') AND (num_elements GT 1)) THEN BEGIN
      FOR j=ndims,1,-1 DO BEGIN
        jstr = STRTRIM(STRING(j), 2)
        axis_name = 'axis' + jstr

; Look for a colname!DEPEND_j variable
        axis_values = ''
        query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
                "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
                " AND UPPER(name)='DEPEND_" + jstr + "'"
        oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
        status = oRS->MoveCursor(/FIRST)
        IF (status EQ 1) THEN BEGIN
          rec = oRS->GetRecord()
          axis_values = rec.value
        ENDIF
        OBJ_DESTROY, oRS

; If found then set up axes_desc as a structure rather than a string
        IF (axis_values NE "") THEN BEGIN

; If a !LABLAXIS parameter was found then we have a simple label with 
; a pointer to another column given by the !DEPEND_j parameter for values 
; of dimension positions

          IF (single_label) THEN BEGIN
            axes_desc = join_struct(axes_desc, $
                                    create_struct(axis_name, $
                                                  {label:axis_label, $
                                                   values:axis_values}))
; Otherwise there should be a !LABL_PTR_j parameter which points to another
; column which contains labels for the dimension positions
          ENDIF ELSE BEGIN
            axis_labels = ''
            query = "SELECT CAST(value AS CHAR) AS value FROM variable_attribute " + $
                    "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
                    " AND UPPER(name)='LABL_PTR_" + jstr + "'"
            oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
            status = oRS->MoveCursor(/FIRST)
            IF (status EQ 1) THEN BEGIN
              rec = oRS->GetRecord()
              axis_labels = rec.value
            ENDIF
            OBJ_DESTROY, oRS
            axes_desc = join_struct(axes_desc, $
                                    create_struct(axis_name, $
                                                  {labels:axis_labels, $
                                                   values:axis_values}))
          ENDELSE
        ENDIF
      ENDFOR
    ENDIF

; Create the metadata structure for this column

    	sedat_make_meta, colname, datatype, ndims, dimensions, null, meta, $
    	    var_type=var_type, units=units, si_rel=si_rel, $
    	    axes_desc=axes_desc, $
    	    status=status, cmn=cmn

; Add the metadata for this column to the structure of all column metadata    

    	if status ne 0 then begin
            print, 'Failed to create metadata for variable '+colname+' in file '+file
    	endif else begin
            varmeta = join_struct(varmeta,create_struct(colname,meta))
    	endelse

  ENDFOR

;  OBJ_DESTROY, oDB

END
