; $Id: sedat_read_record.pro,v 1.9 2001/12/06 12:48:44 stamper Exp $
; 
; sedat_read_record
; 
; Read a data record from an open dataset
; 
; Arguments
;   meta	  I    The dataset metadata structure for the file
;   data          O    A structure containing the data record read
; 
; Keywords 
;   rec           I    The record number to read (optional)
;   status        O    Status code - 0 is success
;   cmn           IO   SEDAT execution environment information
;
; External routines 
;   libidl_stpdf.so STPDF library
;    
; $Log: sedat_read_record.pro,v $
; Revision 1.9  2001/12/06 12:48:44  stamper
; Commented thoroughly.
;
; Revision 1.8  2001/05/09 08:44:19  stamper
; Use the meta.id value to index into the idsid array to get the STPDF file
; number.
;
; Revision 1.7  2000/12/11 15:44:42  stamper
; Added comment header.
; Commented out diagnostic message.
;

pro sedat_read_record, meta, data, rec=rec, status=status, $
                       num_recs=num_recs, ALL=ALL, $
                       cmn=cmn,test=test
;print,'SEDAT_READ_RECORD: Start'
;   start_time = SYSTIME(/SEC)						; ********* TIMING
;   gdsac_time = 0.0D							; ********* TIMING
;   getcol_time = 0.0D							; ********* TIMING

    if n_elements( num_recs) eq 0 then num_recs = 1L

;    data = 0
    status = 0

    ; Set the record number to read if provided

    if (n_elements(rec) eq 1) then meta.rec = rec

    if keyword_set(all) then begin
	num_recs = meta.n_recs
	meta.rec = 1L
    endif

    ; Return with an error if the record number if out of range

    if meta.rec lt 1L or meta.rec gt meta.n_recs then begin
  	status = 1
        print,'*** SEDAT_READ_RECORD: invalid number of records.'
  	return
    endif

    strt_rec = meta.rec
    end_rec = MIN( [meta.n_recs, meta.rec + num_recs - 1L ])
    num_recs = end_rec - meta.rec + 1L

;   Set up the Datastructure from the meta data.
;   if we're only reading a single point and the record structure
;   is defined, then reuse the original record structure.
    if (SIZE(data,/TYPE) eq 8) AND (num_recs eq 1L) then begin
       data = data[0]
       d    = data
    endif else begin
       FOR i=0L,meta.n_vars-1L do begin
  	   varmeta = meta.var.(i)
	   datatype = varmeta.datatype
	   sedat_make_variable,varmeta.dimensions, datatype, value
	   if (datatype eq 'C') then value = ''+value
	   d = join_struct( d, create_struct( varmeta.name, value))
       ENDFOR
       data = REFORM( REPLICATE(d,num_recs))
    endelse

  IF (STRPOS(meta.name, 'SYSTEM!') GE 0) THEN BEGIN

    sedat_read_record_odi, meta, strt_rec, num_recs, data, status
    if status ne 0 then return

  ENDIF ELSE BEGIN

    FOR j=0L, num_recs-1L DO BEGIN
;print,j,'/',num_recs-1, FIX(1.0*j/(num_recs-1)),'%'

	rec_no = j + strt_rec
 	; Read the record
;	   s = sysTIME(/SEC)						; ********* TIMING
	   status = call_external('libidl_stpdf.so','idl_chp_gdsac',$
  			   cmn.idsid[meta.id], rec_no)
;	   gdsac_time = gdsac_time + (SYSTIME(/SEC) - s)		; ********* TIMING
 	   if status ne 0 then return
	   ; For each column in the dataset
	   for i=0L,meta.n_vars-1 do begin
		varmeta = meta.var.(i)
		; Retrieve a value for this variable from the STPDF record read
		; For some reason, this CALL_EXTERNAL won't write to d.(i),
		; but will write to a variable copied from d.(i). So we use inval.
		inval = d.(i)
;		s = SYSTIME(/SEC)					; ********* TIMING
	  	status = call_external('libidl_stpdf.so','idl_chp_getcol', $
	  			       cmn.idsid[meta.id],varmeta.name,$
					inval)
;print,'idl_chp_getcol',varmeta.name
;	  	getcol_time = getcol_time + (SYSTIME(/SEC) - s)		; ********* TIMING
		if varmeta.datatype eq 'C' then d.(i) = '' + inval $
		else d.(i) = inval
	   endfor
	  data[j] = d
    ENDFOR

  ENDELSE

   meta.rec = end_rec + 1L

;total_time = SYSTIME(/SEC) - start_time
;other_time = total_time - gdsac_time - getcol_time
;print,'TOTAL_TIME: ',total_time,' s'
;print,'GDSAC_TIME: ',gdsac_time,' s',gdsac_time/total_time*100.0,'%'		; ********* TIMING
;print,'GETCOL_TIME:',getcol_time, ' s',getcol_time/total_time*100.0,'% ', $
;	' for ', meta.n_vars, ' cols.'					; ********* TIMING
;print,'OTHER:      ',other_time,' s',other_time/total_time*100.0,'%'
end
