<?php

// Parser functions for the ODI database.
// Daniel Heynderickx, DH Consultancy, Belgium
// Peter Wintoft, Swedish Institute of Space Physics


// Function to build a file list of the specified wget directory
  function rfileList($platform, $instrument, $basedir, $filepattern)
  {
    $dfiles = array();

    $fpos = strpos($filepattern, "%");

    $ydirs = scandir($basedir);
    array_shift($ydirs);
    array_shift($ydirs);
    foreach ($ydirs as $ydir)
    {
      if (is_dir($basedir . $ydir))
      {
        $ddir = scandir($basedir . $ydir);
        foreach ($ddir as $dfile)
        {
/*
          $fget = FALSE;
          if ($fpos == 0)
          {
            if (strpos($dfile, substr($filepattern, $fpos+1)) !== FALSE)
              $fget = TRUE;
          }
          else if ($fpos == strlen($filepattern)-1)
          {
            if (strpos($dfile, substr($filepattern, 0, $fpos)) !== FALSE)
               $fget = TRUE;
          }
          else
          {
            if ((strpos($dfile, substr($filepattern, 0, $fpos)) !== FALSE) &&
                (strpos($dfile, substr($filepattern, $fpos+1)) !== FALSE))
              $fget = TRUE;
          }
*/
          $fget = TRUE;
          $parse = explode("%", trim($filepattern, "%"));
          for ($i = 0; $i < count($parse); $i++)
            $fget = $fget && (strpos($dfile, $parse[$i]) !== FALSE);
          if ($fget === TRUE)
          {
            $rfname = $basedir . $ydir . "/" . $dfile;
            $date = stat($rfname);
            $date = gmdate("Y/m/d h:i:s", $date["mtime"]);
            $dfiles[] = array($dfile, $date, $rfname);
          }
        }
      }
    }

// Add the Kyoto HTML pages and Lund forecast page to the file list for Dst
    if ($filepattern == "%dst%")
    {
      $dbasedir = "http://swdc234.kugi.kyoto-u.ac.jp/dst_realtime";
      $findex = file($dbasedir . "/index.html");
      foreach ($findex as $dumstr)
      {
        if (($di = strpos($dumstr, "/index.html")) &&
            (strpos($dumstr, "month") === FALSE) &&
            (strpos($dumstr, "dst") === FALSE))
        {
          $file = $dbasedir . substr($dumstr, $di-7, 18);
          $dum = file($file);
          foreach ($dum as $line)
          {
            $offset = strpos($line, "[Updated");
            if ($offset !== FALSE)
            {
              $date = explode("UT", substr($line, $offset+11));
              $date = date("Y/m/d H:i:s", strtotime($date[0] . ":00"));
              break;
            }
          }
          $dfiles[] = array($file, $date, $file);
        }
      }
      $dfiles[] = array("http://rwc.lund.irf.se/rwc/dst/dst1.txt",
                        date("Y/m/d H:i:s"),
                        "http://rwc.lund.irf.se/rwc/dst/dst1.txt");
    }
// Add the Potsdam last and current month tables and Lund forecast
// page to the file list for KpAp
    if (strpos($filepattern, "%kpap") !== FALSE)
    {
      if (strpos($filepattern, "_3h") !== FALSE)
      {
        $dfiles[] = array("kpap_3h/potsdam/kpyymm.wdc", date("Y/m/d H:i:s"),
                          "http://www-app3.gfz-potsdam.de/kp_index/kpyymm.wdc");
        $dfiles[] = array("kpap_3h/potsdam/pqlyymm.wdc", date("Y/m/d H:i:s"),
                          "http://www-app3.gfz-potsdam.de/kp_index/pqlyymm.wdc");
        $dfiles[] = array("kpap_3h/lund/kp0.txt", date("Y/m/d H:i:s"),
                          "http://rwc.lund.irf.se/rwc/kp/kp0.txt");
      }
      else
      {
        $dfiles[] = array("kpap_1d/potsdam/kpyymm.wdc", date("Y/m/d H:i:s"),
                          "http://www-app3.gfz-potsdam.de/kp_index/kpyymm.wdc");
        $dfiles[] = array("kpap_1d/potsdam/pqlyymm.wdc", date("Y/m/d H:i:s"),
                          "http://www-app3.gfz-potsdam.de/kp_index/pqlyymm.wdc");
      }
    }

    return $dfiles;
  }


// Parse a data file and write formatted content to load.tmp
  function parseFile($rname, $fid, $platform, $instrument, $sk_file)
  {
    $result = FALSE;

    if (strpos(strtolower($rname), ".cdf") !== FALSE)
    {
      $sname = substr_replace($sk_file, "set", -3, 3);
      if ((strpos($rname, "SREMRosetta") !== FALSE) ||
          ((strpos($rname, "IREM") !== FALSE) && (strpos($rname, "_exp") === FALSE) && (strpos($rname, "_pow") === FALSE)))
      {
        @unlink("tmp.cdf");
        exec("gunzip -c " . $rname . " > tmp.cdf");
        $rname = "tmp.cdf";
      }
      exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings " . $sname . " -initial \"noheading,nooutput\" -text tmp.txt " . $rname);

      if (file_exists("tmp.txt"))
      {
        $tfile = fopen("load.tmp", "w");
        $ihandle = fopen("tmp.txt", "r");
        while (!feof($ihandle))
        {
          $sdum = fgets($ihandle);
          $sdum = preg_split("/\s+/", $sdum);
          if (count($sdum) > 2)
          {
            $ts = array_shift($sdum) . " " . array_shift($sdum);
            $dpos = strpos($ts, ".");
            if ($dpos !== FALSE)
              $ms = intval(substr($ts, $dpos+1));
            else
              $ms = 0;
            $ts = strtotime($ts);
            $epoch = date("Y-m-d H:i:s", $ts);
            $cdf_epoch = date_to_cdfepoch($epoch, $ms);
            fprintf($tfile, "%20.3f, %s, %d", $cdf_epoch, $epoch, $ms);
            for ($i = 0; $i < count($sdum)-1; $i++)
            {
//              if ($sdum[$i] < -1.0E20)
//                fprintf($tfile, ",NULL");
//              else
                fprintf($tfile, ",%s", $sdum[$i]);
            }
            fprintf($tfile, ", %d%s", $fid, LF);
          }
        }
        fclose($tfile);
        fclose($ihandle);
        @unlink("tmp.txt");

        $result = TRUE;
      }
      else
        $result = FALSE;
    }
    else if (strpos(strtolower($rname), "dst") !== FALSE)
    {
      $ms = 0;
      $dsts = file($rname);
      $tfile = fopen("load.tmp", "w");
      if ($dsts !== FALSE)
      {
        if (strpos($rname, "lund") !== FALSE)
        {
          $dum = preg_split("/\s+/", trim($dsts[0]));
          $date = $dum[0];
          $hour = substr($dum[1], 0, 2);
          $epoch = $date . " " . $hour . ":00:00";
          $dstval = floatval($dum[2]);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);
          fprintf($tfile, "%20.3f, %s, %d, %d,% d, %d%s",
                          $cdf_epoch, $epoch, $ms, $dstval, 3, $fid, LF);
        }
        else if (strpos($rname, "kyoto") !== FALSE)
        {
          $di = strpos($rname, "/index.html");
          $year = substr($rname, $di-6, 4);
          $month = substr($rname, $di-2, 2);
//          $ndays = cal_days_in_month(CAL_GREGORIAN, $month, $year);
          $ndays = intval(strftime("%e", mktime(0, 0, 0, $month+1, 0, $year)));
          $nq = intval(floor($ndays/5));
          $nr = $ndays - $nq * 5;
          $dumstr = array_shift($dsts);
          while (strpos($dumstr, "DAY") === FALSE)
            $dumstr = array_shift($dsts);
          for ($i = 0; $i < $nq*6+$nr; $i++)
          {
            $dumstr = array_shift($dsts);
            if (($i+1) % 6 > 0)
            {
              $day = sprintf("%02d", substr($dumstr, 0, 2));
              for ($j = 0; $j < 3; $j++)
              {
                for ($k = 0; $k < 8; $k++)
                {
                  $epoch = $year . "-" . $month . "-" . $day . " " .
                           sprintf("%02d", $j*8+$k) . ":00:00";
                  $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                  $dstval = intval(substr($dumstr, 3+$j*33+$k*4, 4));
                  if ($dstval == 9999) $dstval = "NULL";
                  fprintf($tfile, "%20.3f, %s, %d,%s, %d, %d%s",
                                  $cdf_epoch, $epoch, $ms, $dstval, 2, $fid, LF);
                }
              }
            }
          }
        }
        else
        {
          foreach ($dsts as $dst)
          {
            $year = intval(substr($dst, 3, 2));
            if ($year > 30) $year += 1900; else $year += 2000;
            $month = intval(substr($dst, 5, 2));
            if ($month > 0)
            {
              $day = intval(substr($dst, 8, 2));
              $flag = substr($dst, 7, 1);
              if ($flag == "*") $flag = 0; else $flag = 1;
              for ($i = 0; $i < 24; $i++)
              {
                $dstval = intval(substr($dst, 20+$i*4, 4));
                $epoch = sprintf("%04d-%02d-%02d %02d:00:00", $year, $month, $day, $i);
                $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                fprintf($tfile, "%20.3f, %s, %d, %d, %d, %d%s",
                                $cdf_epoch, $epoch, $ms, $dstval, $flag, $fid, LF);
              }
            }
          }
        }
        fclose($tfile);
        $result = TRUE;
      }
      else
        $result = FALSE;
    }
// Parse Kp and Ap.
    else if (($sk_file == "KpAp_3h.skt") || ($sk_file == "KpAp_1d.skt"))
    {
      $ms = 0;
      $kpaps = file($rname);
      $tfile = fopen("load.tmp", "w");
      if ($kpaps !== FALSE)
      {
        if ($sk_file == "KpAp_3h.skt")        {
// Predicted Kp from Lund
          if (strpos($rname, "lund") !== FALSE)
          {
            $dum = preg_split("/\s+/", trim($kpaps[0]));
            $date = $dum[0];
            $hour = substr($dum[1], 0, 2);
            $epoch = $date . " " . $hour . ":00:00";
            $kpval = floatval($dum[2]);
            $cdf_epoch = date_to_cdfepoch($epoch, $ms);
            fprintf($tfile, "%20.3f, %s, %d, %f, %d,NULL,NULL, %d%s",
                            $cdf_epoch, $epoch, $ms, $kpval, 3, $fid, LF);
          }
// 3h Kp and Ap from NGDC and Potsdam
          else
          {
            if (strpos($rname, "qlyymm") !== FALSE)
              $flag = 2;
            else
              $flag = 0;
            foreach ($kpaps as $kpap)
            {
              $year = intval(substr($kpap, 0, 2));
              if ($year > 30) $year += 1900; else $year += 2000;
              $month = intval(substr($kpap, 2, 2));
              $day = intval(substr($kpap, 4, 2));
              for ($i = 0; $i < 8; $i++)
              {
                $hour = $i * 3;
                $epoch = sprintf("%04d-%02d-%02d %02d:00:00", $year, $month, $day, $hour);
                $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                $kpd = intval(substr($kpap, 12+$i*2, 2));
                $kpq = floor($kpd/10);
                $kpr = $kpd % 10;
                switch ($kpr)
                {
                  case 0:
                    $kp = $kpq;
                    break;
                  case 3:
                    $kp = $kpq + 1.0 / 3.0;
                    break;
                  case 7:
                    $kp = $kpq + 2.0 / 3.0;
                    break;
                }
                $ap = intval(substr($kpap, 31+$i*3, 3));
                fprintf($tfile, "%20.3f, %s, %d, %f, %d, %d, %d, %d%s",
                                $cdf_epoch, $epoch, $ms, $kp, $flag, $ap, $flag, $fid, LF);
              }
            }
          }
        }
        else
        {
// Daily values from NGDC and Potsdam
          if (strpos($rname, "qlyymm") !== FALSE)
            $flag = 2;
          else
            $flag = 0;
          foreach ($kpaps as $kpap)
          {
            $year = intval(substr($kpap, 0, 2));
            if ($year > 30) $year += 1900; else $year += 2000;
            $month = intval(substr($kpap, 2, 2));
            $day = intval(substr($kpap, 4, 2));
            $epoch = sprintf("%04d-%02d-%02d 00:00:00", $year, $month, $day);
            $cdf_epoch = date_to_cdfepoch($epoch, $ms);
            $ap = substr($kpap, 55, 3);
            $cp = substr($kpap, 58, 3);
            $c9 = substr($kpap, 61, 1);
            fprintf($tfile, "%20.3f, %s, %d, %s, %d, %s, %d, %s, %d",
                            $cdf_epoch, $epoch, $ms, $ap, $flag, $cp, $flag, $c9, $flag);

            if (strpos($rname, "potsdam") === FALSE)
              fprintf($tfile, ",%s, %d, %s, %s", substr($kpap, 62, 3), $flag,
                              substr($kpap, 65, 5), substr($kpap, 70, 1));
            else
              fprintf($tfile, ",NULL,NULL,NULL,NULL");
            fprintf($tfile, ",%d%s", $fid, LF);
          }
        }
        fclose($tfile);
        $result = TRUE;
      }
      else
        $result = FALSE;
    }
// Parse GOES files
    else if ((substr($rname, -4) == ".TXT") &&
             ((substr($rname, -9, 1) == "5") || (substr($rname, -9, 1) == "1")) &&
             ((substr($rname, -12, 1) == "A") || (substr($rname, -12, 1) == "G") ||
              (substr($rname, -12, 1) == "H") || (substr($rname, -12, 1) == "I") ||
              (substr($rname, -12, 1) == "Z")))
    {
// Read in GOES longitude file
      $longs = file("http://goes.ngdc.noaa.gov/docs/GoesLongitudes.txt");
      while (strpos($longs[0], "------") === FALSE) array_shift($longs);
      array_shift($longs);

      $gnr = substr($rname, -11, 2);
      $yymm = substr($rname, -8, 4);
      $glon = array();
      for ($i = 0; $i < 31; $i++) $glon[] = "0.0";
      foreach ($longs as $long)
      {
        if (substr($long, 5, 7) == $gnr . " " . $yymm)
        {
          $glon = preg_split("/[\s]+/", trim(substr($long, 12)));
          break;
        }
      }

      $ms = 0;
      $goes = file($rname);
      if ($goes !== FALSE)
      {
        $efile = fopen("epoch.tmp", "w");
        $vfile = fopen("values.tmp", "w");
        while (strpos($goes[0], "------") === FALSE) array_shift($goes);
        array_shift($goes);
        foreach ($goes as $sdum)
        {
          list($year, $month, $day, $hour, $min) = sscanf($sdum, "%2d%2s%2s %2s%2s");
          if ($year < 50) $year += 2000; else $year += 1900;
          $epoch = sprintf("%04d-%02d-%02d %02d:%02d:00", $year, $month, $day, $hour, $min);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);

          $xrays = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 15, 20))));
          $magc = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 35, 27))));
          $mags = trim(substr($sdum, 62, 9));
          $el = trim(substr($sdum, 71, 10));
          $pr = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 81))));

          fprintf($vfile, "%s,%s,%s,%s,%s, %d%s",
                          $xrays, $magc, $mags, $el, $pr, $fid, LF);
          fprintf($efile, "%20.3f, %s, %3d, %7.2f%s",
                          $cdf_epoch, $epoch, $ms, $glon[$day-1], LF);
        }
        fclose($vfile);
        fclose($efile);
// Add magnetic coordinates
        exec($_SERVER["ODI_HOME"] . "/parsers/goes_bl");
        $pos = file("position.tmp", FILE_IGNORE_NEW_LINES);
        $vals = file("values.tmp", FILE_IGNORE_NEW_LINES);
        $tfile = fopen("load.tmp", "w");
        for ($i = 0; $i < count($pos); $i++)
          fprintf($tfile, "%s, %s%s", $pos[$i], $vals[$i], LF);
        fclose($tfile);

        @unlink("epoch.tmp");
        @unlink("position.tmp");
        @unlink("values.tmp");

        $result = TRUE;
      }
      else
        $result = FALSE;
    }
    // Parse SSN_1m files
    else if ($sk_file == "SSN_1m.skt")
    {
      $ms = 0;
      $ssn = file($rname);
      $tfile = fopen("load.tmp", "w");
      if ($ssn !== FALSE)
      {
        foreach ($ssn as $ssn_row)
        {
          list($year,$month,$y,$s,$s12) = sscanf($ssn_row,"%4d%2d %f %f %f");
          $epoch = sprintf("%04d-%02d-01 00:00:00", $year, $month);
          $cdf_epoch = date_to_cdfepoch($epoch, $ms);

          fprintf($tfile, "%20.3f, %s, %d, " .
                          "%5.1f, %5.1f, " .
                          "%d%s",
                           $cdf_epoch, $epoch, $ms,
                           $s, $s12,
                           $fid, LF);
        }
        fclose($tfile);

        $result = TRUE;
      }
      else
        $result = FALSE;
    }
    return $result;
  }



?>