#!/usr/bin/php
<?php
// Check the skeleton file.
// Peter Wintoft, Swedish Institute of Space Physics
// 2009-09-08

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("sk_file_name");
$args=get_cl_arguments($argv,$arg_array);

// Get the cleaned skeleton.
$row_all = clean_skeleton($args["sk_file_name"]);

// Loop over all rows, interpret, and store into database.
$block="";          // Block flag: head, global, zvariable, ...
$in_block=FALSE;    // True when in a block.
$nrv_index=0;       // Index used to count nrv values when they exist.
$num_global_att=0;  // The number of global attributes listed in the GLOBALattributes block.
$num_rVariable=0;
$num_zVariable=0;
$rVar = array();
$zVar = array();
$var_att_list=array();  // The listed variable attributes (#VARIABLEattributes).
foreach($row_all as $row) {
    // Get lines starting with "#" indicating start of block.
    if(preg_match("/#(\w+)/",$row,$match)>0) {
        $block=$match{1};
        #But still not in block.
        $in_block=FALSE;
    }
    // Read data if in a block.
    if($in_block) {
        // Check type of block.
        switch($block) {
            // header block.
            case "header":
                // Get the variables count line.
                if(preg_match_all('/(\d+)\/(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\/z\s+(\d+)(.*)/',$row,$match)) {
                    $variable_count=array();
                    $variable_count["rVariables"]=$match[1][0];
                    $variable_count["zVariables"]=$match[2][0];
                    $variable_count["gAttributes"]=$match[3][0];
                    $variable_count["vAttributes"]=$match[4][0];
                    $variable_count["records"]=$match[5][0];
                    $variable_count["dims"]=$match[6][0];
                    if(isset($match[7][0])) {
                        $variable_count["sizes"]=trim($match[7][0]);
                    }
                    else {
                        $variable_count["sizes"]=NULL;
                    }
                    fwrite(STDOUT,"===================\nVariable count line\n=============\n");
                    foreach($variable_count as $key=>$value) {
                        fwrite(STDOUT,"Number of ".$key.": ".$value."\n");
                    }
                }
                break;
            // Global attributes block.
            case "GLOBALattributes":
                // Count the number of global attributes.
                if(preg_match('/".+"\s+\d+:\s+.+\s+\{\s+".*"/',$row))
                    $num_global_att++;
                break;
            // The listed variable attributes.
            case "VARIABLEattributes":
                // Get list of variable attributes.
                if(preg_match('/"(.+)"/',$row,$match)) {
                    $var_att_list[] = array(
                        "attribute"=>no_quote($match[1]),
                        "listed"=>TRUE,
                        "used"=>FALSE);
                }
            // rVariable block.
            case "variables":
                // rVariable.
                if(preg_match_all('/"(.+)"\s+(\w+)\s+(\d+)\s+(\w)(.*)/',$row,$match)) {
                    $var_name = no_quote($match[1][0]);
                    $rVar[$var_name] = array(
//                        "name"=>$match[1][0],
                        "data_type"=>$match[2][0],
                        "number_elements"=>$match[3][0],
                        "record_variance"=>$match[4][0],
                        "dimension_variances"=>str_replace(" ",",",trim($match[5][0])),
                        "dims"=>$variable_count["dims"],
                        "sizes"=>$variable_count["sizes"]);
                    $nrv_index=0;
                }
                // Get variable attributes.
                if(preg_match_all('/("\w+")\s+(\w+)\s+\{\s+(.*)\s+\}/',$row,$match)) {
                    $varatt[$var_name][] = array(
//                    $varatt["variable_metadata"]=$var["name"];
                        "attribute"=>no_quote($match[1][0]),
                        "data_type"=>$match[2][0],
                        "value"=>$match[3][0]);
                }
                // Get nrv values.
                if(preg_match_all('/^\[(.*)\]\s*=\s*\{*\s*(.*)\s*\}*/',$row,$match)) {
                    // Remove a possible trailing "}".
                    $nrv_index++;
                    $nrv_pos=trim($match[1][0]," }");
                    $nrv_value=trim($match[2][0]," }");
                    $nrv[$var_name][] = array(
                        "index"=>$nrv_index,
                        "pos"=>$nrv_pos,
                        "value"=>$nrv_value);
                }
                break;
            // zVariable block.
            case "zVariables":
                // Variable name row always contain "Number Elements" and "Dims".
                if(preg_match_all('/"(.+)"\s+(\w+)\s+(\d+)\s+(\d+)\s+(.*)/',$row,$match)) {
                    $var_name = no_quote($match[1][0]);
                    $zVar[$var_name] = array(
//                        "dataset"=>$dataset_id,
//                        "name"=>$match[1][0],
                        "data_type"=>$match[2][0],
                        "number_elements"=>$match[3][0],
                        "dims"=>$match[4][0]);
                    $rest=$match[5][0];
                    if($zVar[$var_name]["dims"]>0) {
                        $zVar[$var_name]["sizes"]="";
                        for($i=0;$i<$zVar[$var_name]["dims"];$i++) {
                            preg_match_all('/(\d+)\s+(.*)/',$rest,$match);
                            $zVar[$var_name]["sizes"] .= $match[1][0].",";
                            $rest=$match[2][0];
                        }
                        $zVar[$var_name]["sizes"]=trim($zVar[$var_name]["sizes"],",");
                        $rest=$match[2][0];
                    }
                    else {
                        $zVar[$var_name]["sizes"]="";
                    }
                    if(preg_match_all('/(\w)\s+(.*)/',$rest,$match)) {
                        $zVar[$var_name]["record_variance"]=$match[1][0];
                        $zVar[$var_name]["dimension_variances"]=str_replace(" ",",",trim($match[2][0]));
                    }
                    else {
                        $zVar[$var_name]["record_variance"]=$rest;
                        $zVar[$var_name]["dimension_variances"]="";
                    }
                    $nrv_index=0;
                }
                // Get variable attributes.
                if(preg_match_all('/("\w+")\s+(\w+)\s+\{\s+(.*)\s+\}/',$row,$match)) {
                    $varatt[$var_name][] = array(
//                    $varatt["variable_metadata"]=$var["name"];
                        "attribute"=>no_quote($match[1][0]),
                        "data_type"=>$match[2][0],
                        "value"=>$match[3][0]);
                }
                // Get nrv values.
                if(preg_match_all('/^\[(.*)\]\s*=\s*\{*\s*(.*)\s*\}*/',$row,$match)) {
                    // Remove a possible trailing "}".
                    $nrv_index++;
                    $nrv_pos=trim($match[1][0]," }");
                    $nrv_value=trim($match[2][0]," }");
                    $nrv[$var_name][] = array(
                        "index"=>$nrv_index,
                        "pos"=>$nrv_pos,
                        "value"=>$nrv_value);
                }
                break;
        } // End switch.
    }
    $in_block=TRUE;
}

// Global attribute counting finished when we reach this block.
fwrite(STDOUT,"==========================================\n");
fwrite(STDOUT,"Number of listed global attributes: ".$num_global_att.".");
if($num_global_att==$variable_count["gAttributes"]) {
    fwrite(STDOUT," OK!\n");
}
else {
    fwrite(STDOUT," Number of gAttributes and number of listed global attributes does not match!\n");
}

fwrite(STDOUT,"==========================================\n");
fwrite(STDOUT,"Number of listed variable attributes: ".count($var_att_list).".");
if(count($var_att_list)==$variable_count["vAttributes"]) {
    fwrite(STDOUT," OK!\n");
}
else {
    fwrite(STDOUT," Number of vAttributes and number of listed variable attributes does not match!\n");
}

fwrite(STDOUT,"==========================================\n");
fwrite(STDOUT,"Number of defined rVariables: ".count($rVar).".");
if(count($rVar)==$variable_count["rVariables"]) {
    fwrite(STDOUT," OK!\n");
}
else {
    fwrite(STDOUT," Number of rVariables and number of defined r-Variables does not match!\n");
}

fwrite(STDOUT,"==========================================\n");
fwrite(STDOUT,"Number of defined zVariables: ".count($zVar).".");
if(count($zVar)==$variable_count["zVariables"]) {
    fwrite(STDOUT," OK!\n");
}
else {
    fwrite(STDOUT," Number of zVariables and number of defined z-Variables does not match!\n");
}

$var_att_not_in_list = array();
foreach($varatt as $var) {
    foreach($var as $att) {
        $in_list = FALSE;
        for($i=0;$i<count($var_att_list);$i++) {
            if(strcasecmp($att["attribute"],$var_att_list[$i]["attribute"])==0) {
                $var_att_list[$i]["used"]=TRUE;
                $in_list = TRUE;
            }
        }
        if(!$in_list)
            $var_att_not_in_list[] = $att["attribute"];
    }
}
$var_att_not_in_list = array_unique($var_att_not_in_list);

fwrite(STDOUT,"==========================\nChecking listed variable attributes.\n");
$max_strlen=0;
foreach($var_att_list as $att) {
    $n=strlen($att["attribute"]);
    if($n>$max_strlen)
        $max_strlen = $n;
}
foreach($var_att_not_in_list as $att) {
    $n=strlen($att);
    if($n>$max_strlen)
        $max_strlen = $n;
}
foreach($var_att_list as $att) {
    printf("%".$max_strlen."s :",$att["attribute"]);
    if($att["used"])
        fwrite(STDOUT," Used.\n");
    else
        fwrite(STDOUT," Not used.\n");
}
foreach($var_att_not_in_list as $att) {
    printf("%".$max_strlen."s : Not listed.".LF,$att);
}
?>
