#!/usr/bin/php
<?php
/**
* Export ODI data to text file.
*
* This program exports ODI data to a text file. The dataset, epoch range,
* and variables to be exported must be defined.
* @author Peter Wintoft <peter@lund.irf.se>
* @version 1.0
* @package odi
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("export_settings_file");
$args=get_cl_arguments($argv,$arg_array);

// Load the XML export_file as a DOM Document.
$dom = new DOMDocument;
$dom->load($args["export_settings_file"]);

// Connect to the database at user level 3.
$conn_array=get_conn_array(3);
$conn=connect_to_database_array($conn_array);

// Get the export file name and write header.
$list=$dom->getElementsByTagName("export_file");
$export_file_name = trim($list->item(0)->nodeValue);
$export_file = fopen($export_file_name,"w");
$list=$dom->getElementsByTagName("creator");
$creator_name = trim($list->item(0)->nodeValue);
$creator_date = gmdate("Y-m-d H:i:s");
fwrite($export_file,"<?xml version=1.0?>".LF);
fwrite($export_file,"<odi_data_export>".LF);
fwrite($export_file,"<creator>".$creator_name."</creator>".LF);
fwrite($export_file,"<date_created>".$creator_date."</date_created>".LF);
fwrite($export_file,"<settings_file>".$args["export_settings_file"]."</settings_file>".LF);

// Get the dataset name.
$list=$dom->getElementsByTagName("dataset");
$dataset = trim($list->item(0)->nodeValue);

// Get the start and end epochs.
$list=$dom->getElementsByTagName("epoch_start");
$epoch = trim($list->item(0)->nodeValue);
list($date,$ms) = explode(".",$epoch);
$epoch_start_cdf = date_to_cdfepoch($date,$ms);
$list=$dom->getElementsByTagName("epoch_end");
$epoch = trim($list->item(0)->nodeValue);
list($date,$ms) = explode(".",$epoch);
$epoch_end_cdf = date_to_cdfepoch($date,$ms);

// Get max number of records.
$list=$dom->getElementsByTagName("max_num_rec");
$max_num_rec = trim($list->item(0)->nodeValue);

// Get the variable names.
$list=$dom->getElementsByTagName("variable");
$field=array();
foreach($list as $l) {
    $field[] = trim($l->nodeValue);
}

// Get the metadata for each variable.
$meta = get_variable_metadata($conn,$dataset,$field);

// Select columns in dataset_* and query. Get the field names.
list($res,$field_name) = get_data_and_fields($conn,$dataset,$epoch_start_cdf,$epoch_end_cdf,
                            $max_num_rec,$meta);

fwrite($export_file,"<field_descriptor>".LF);
for($i=1;$i<=count($field_name);$i++) {
    fwrite($export_file,"<num>".$i."</num><label>".$field_name[$i-1]."</label>".LF);
}
fwrite($export_file,"</field_descriptor>".LF);

$n = $res->num_rows;
fwrite($export_file,"<data num_records=".quote($n).">".LF);
while($row=$res->fetch_row()) {
    fwrite($export_file,implode(", ",$row).LF);
}

fwrite($export_file,"</data>".LF);
fwrite($export_file,"</odi_data_export>".LF);

$conn->close();
fclose($export_file);

// $dom->load($export_file);
// $list=$dom->getElementsByTagName("data");
?>